/* ************************************************
*  GVAR Modelling:
*  An Application to KOREAN ECONOMY
*  --------------------------------
*  Written by Yongcheol Shin. Last updated November 2007
*  Contact to Y.SHIN@LEEDS.AC.UK
*  -------------------------------------------------
*  See Pesaran and Shin (1996, JoE, 1998, EL, 2003, ER),
*  GLPS (2003a, EJ, 2003b, JASA, 2006, OUP), PSW (2004), DdPS (2007),
*  Shin (2009) and GHS (2010)
*
*  This program combines the estimation results of individual country-specific  
*  conditional VECM and construct the GVAR using the link matrices. 
*
*  Currently, there are 26 countries/regions in the following order:
*
	1 USA 
	2 Eurozone
	3 Japan
	4 UK
	5 Norway
	6 Sweden
	7 Switzerland
	8 Canada
	9 Australia
	10 New Zealand
	11 SAfrica
	12 Argentina
	13 Brazil
	14 Chile
	15 Mexico
	16 India
	17 Korea
	18 Malaysia
	19 Philippines
	20 Singapore
	21 Thailand
	22 China (no q)
	23 Indonesia (no q)
	24 Peru (no q)
	25 Turkey (no q)
	26 Saudi Arabia (no q and no r)
*
*  This order is purely made for convenience.
*
*  But notice that the individual country-specific estimation is based on 
*  the following specification:  
*
*  US: There are 7 endogenous variables (po,r,im,ex,q,dp,yd) 
*      and 3 exogenous (foreign) variables (eps,dps,ys)
*
*  EU - Thailand (20 countries):
*      There are 7 endogenous variables (re,r,im,ex,q,dp,yd)  
*      and 5 exogenous (foreign) variables (po,rs,qs,dps,ys)
*
*  China - Turkey (4 countries):
*      There are 6 endogenous variables (re,r,im,ex,dp,yd)  
*      and 5 exogenous (foreign) variables (po,rs,qs,dps,ys)
*
*  Saudi Arabia: There are 5 endogenous variables (re,im,ex,dp,yd)  
*      and 5 exogenous (foreign) variables (po,rs,qs,dps,ys)
*
*  The variables are defined as below:
*
*  yd: log real output
*  p: log CPI
*  dp: inflation 
*  q: log real stock price index 
*  e: log nominal exchange rate (local currency/$) 
*  r: short-temr interest rate
*  h: log (money/output)
*  ex: log real export 
*  im: log real import 
*  po: log oil price, Dubai 
*  Then all the foreign variables are constructed as the trade-weight-based average
*  rs: foreign interest 
*  qs: log foreign stock price index
*  dps: foreign inflation
*  ys: log foreign output
*  es: log foreign nominal exchange rates
*  ps: log foreign CPI
*  Then construct the real (effective) exchange rates as
*  pps = p-ps: relative log price
*  ees = e-es: nominal effective exchange rate 
*  ep = e-p: real exchange rate 
*  eps = es-ps: Foreign real (effective) exchange rate
*  re = ep-eps; real (effective) exchange rate
*
*  For the data see the procedure DATA1
*
*  These differences are due to the data availability and the US' special role 
*  in the global economy. See GHS (2007) for more details.
*
*  This program consists of the following steps:
*
*  1. The initial step loads and defines the data. It also specifies 
*     some initial informations necessary for running the rest of the program.
*
*  2. The country-specifc VECM estimation results using the r(i) exactly-
*     identified cointegrating relationships. 
*
*  3. GVAR estimation results including:
*
*     a: Persistence profiles (speed of convergence of LR relationships)
*     b: Generalized impulse response functions
*     c: Generalized forecast error decompositions
*
*     We plan to address the model uncertainty in the evaluation of forecasting in future.
* ************************************************************************************** */
new;  cls; library pgraph;  graphset; time;
  
@ Specify the output file @
output file = GVAR_A.out reset;

@* Some preliminary information necessary for running the program @

porder = 2;      @ VAR lag order @

x_op = 2;        @ marginal VAR option for dx: 1 with break, 2 without @
nhorz = 24;      @ number of forecast horizons @
n_cou = 26;      @ No. of countries @
stt0 = 0;        @ number of initial observations discarded @ 
                 @ so the current sample starts from 1982q1 @ 
stt1 = 8;        @ number of last observations discarded @
                 @ you may delete a few obs due to the data revision error @ 

@ Individual country-specific information on Casei, brpointi and rcoini
* *********************************************************************
* casei: 7 cases of deterministic option for Cointegrating VAR
         1. No intercepts and no trends.
         2. Restricted intercepts and no trends.
         3. Unrestricted intercepts and no trends.
         4. Unrestricted intercepts and restricted trends.
         5. Unrestricted intercepts and unrestricted trends.
         6. Case 4 plus restricted intercept shift dummies.
         7. Case 4 plus restricted trend shift dummies. 
*
* brpointi: known break point
* rcoini: # of cointegrating relations 
*
* These require the preliminary estimation results. 
* Here you need to choose mainly between Case 4 and Case 6. 
*
* To this end a careful analysis of the structural break should be made 
* prior to estimation. Currently, only the single break point is allowed.
*
* Once the appropriate case is selected, then you will find the testing
* results for the number of cointegrating relations from the file, 
* CoinTest.xls generated by the program. 
*
* Each of three information is 26 by 1 vector and must be saved in 
* GVARSUMMARY.xls
*
* Once this is done, run the program again.
* ***************************************** @

CASEi = xlsreadm("GVARSUMMARY","C2:C27",1,""); 
BRPOINTi = xlsreadm("GVARSUMMARY","D2:D27",1,""); 
RCOINi = xlsreadm("GVARSUMMARY","E2:E27",1,""); 


@ ****************************
* Generalised Impulse Response
* **************************** @
@ Candidate shocks: 
-----------------------------
----- original shocks -------
-----------------------------
1. US positive real GDP shock: 7
2. EU positive real GDP shock: 7 + 7 = 14   
3. JPY positive real GDP shock: 2*7 + 7 = 21   
4. CHN positive real GDP shock: 21*7 + 6 = 153   
5. US positive equity price shock: 5
6. EU positive equity price shock: 7 + 5 = 12   
7. JPY positive equity price shock: 2*7 + 5 = 19   
8. US positive import shock: 3
9. CHN positive export shock: 21*7 + 4 = 151

------------------------------------------------------
----- combined shock (by adding or subtracting or change in sign) -------
------------------------------------------------------
10. Concerted simultaneous real GDP shock in US, EU, JPN, CHN
11. Concerted simultaneous equity price shock in US, EU, JPN
12. Combination of positive US import shock and positive CHN export shock.
* ***************************** @

nshock = 9; @ number of shocks @
eshock = zeros(nshock,1); @ shock selection vector, see above @
eshock[1]  = 7;    
eshock[2]  = 7 + 7;    
eshock[3]  = 2*7 + 7;
eshock[4]  = 21*7 + 6;    
eshock[5]  = 5; 
eshock[6]  = 7+5;
eshock[7]  = 2*7+5;
eshock[8]  = 3; 
eshock[9]  = 21*7+4;



@ *************************************************
* Generalised Forecast Error Variance Decomposition
* ************************************************* @
@ Candidate shocks: 
-----------------------------
----- original shocks -------
-----------------------------
1. Oil price shock: 1
2. US equity price shock: 5
3. EU output shock : 7 + 7 = 14   
4. US nominal interest rate shock: 2
5. EU nominal interest rate shock: 7 + 2 = 9
6. JAP nominal interest rate shock: 2*7 + 2 = 16
7. UK nominal interest rate shock:  3*7 + 2 = 23
8. CHN nominal interest rate shock: 21*7 + 2 = 149
9. US inflation shock: 6
10. EU inflation shock: 7 + 6 = 13
11. JAP inflation shock: 2*7 + 6 = 20
12. UK inflation shock:  3*7 + 6 = 27
13. CHN inflation shock: 21*7 + 5 = 152
14. EU real exch rate: 7 + 1 = 8
15. JAP real exch rate: 2*7 + 1 = 15
16. UK real exch rate:  3*7 + 1 = 22
17. CHN real exch rate: 21*7 + 1 = 148
* ***************************** @

nvshock = 17; @ number of variables for FEVD @
vshock = zeros(nvshock,1); @ variable selection vector, see above @
vshock[1]  = 1;    
vshock[2]  = 5;    
vshock[3]  = 7 + 7;
vshock[4]  = 2;    
vshock[5]  = 7+2; 
vshock[6]  = 2*7+2;
vshock[7]  = 3*7+2;
vshock[8]  = 21*7+2; 
vshock[9]  = 6; 
vshock[10] = 7+6;
vshock[11] = 2*7+6;
vshock[12] = 3*7+6;
vshock[13] = 21*7+5;
vshock[14] = 7+1;
vshock[15] = 2*7+1;
vshock[16] = 3*7+1;
vshock[17] = 21*7+1;

@ Headers for Excel files @
chd1 = {"US" "US" "US" "US" "US" "US" "US"};
chd2 = {"EU" "EU" "EU" "EU" "EU" "EU" "EU"};
chd3 = {"JAP" "JAP" "JAP" "JAP" "JAP" "JAP" "JAP"};
chd4 = {"UK" "UK" "UK" "UK" "UK" "UK" "UK"};
chd5 = {"NOR" "NOR" "NOR" "NOR" "NOR" "NOR" "NOR"};
chd6 = {"SWE" "SWE" "SWE" "SWE" "SWE" "SWE" "SWE"};
chd7 = {"SWI" "SWI" "SWI" "SWI" "SWI" "SWI" "SWI"};
chd8 = {"CAN" "CAN" "CAN" "CAN" "CAN" "CAN" "CAN"};
chd9 = {"AUS" "AUS" "AUS" "AUS" "AUS" "AUS" "AUS"};
chd10 = {"NZ" "NZ" "NZ" "NZ" "NZ" "NZ" "NZ"};
chd11 = {"SAF" "SAF" "SAF" "SAF" "SAF" "SAF" "SAF"};
chd12 = {"ARG" "ARG" "ARG" "ARG" "ARG" "ARG" "ARG"};
chd13 = {"BRA" "BRA" "BRA" "BRA" "BRA" "BRA" "BRA"};
chd14 = {"CHL" "CHL" "CHL" "CHL" "CHL" "CHL" "CHL"};
chd15 = {"MEX" "MEX" "MEX" "MEX" "MEX" "MEX" "MEX"};
chd16 = {"IND" "IND" "IND" "IND" "IND" "IND" "IND"};
chd17 = {"KOR" "KOR" "KOR" "KOR" "KOR" "KOR" "KOR"};
chd18 = {"MAL" "MAL" "MAL" "MAL" "MAL" "MAL" "MAL"};
chd19 = {"PHI" "PHI" "PHI" "PHI" "PHI" "PHI" "PHI"};
chd20 = {"SIN" "SIN" "SIN" "SIN" "SIN" "SIN" "SIN"};
chd21 = {"THA" "THA" "THA" "THA" "THA" "THA" "THA"};
chd22 = {"CHN" "CHN" "CHN" "CHN" "CHN" "CHN"};
chd23 = {"INDO" "INDO" "INDO" "INDO" "INDO" "INDO"};
chd24 = {"PER" "PER" "PER" "PER" "PER" "PER"};
chd25 = {"TUR" "TUR" "TUR" "TUR" "TUR" "TUR"};
chd26 = {"SAU" "SAU" "SAU" "SAU" "SAU"};
CHD = CHD1~CHD2~CHD3~CHD4~CHD5~CHD6~CHD7~CHD8~CHD9~CHD10
     ~CHD11~CHD12~CHD13~CHD14~CHD15~CHD16~CHD17~CHD18~CHD19~CHD20
     ~CHD21~CHD22~CHD23~CHD24~CHD25~CHD26;

header0 = {"po" "r" "im" "ex" "q" "p" "y"};
header1a = {"ep" "r" "im" "ex" "q" "dp" "y"};
hj = 1;
do until hj > 20;
   if hj eq 1; header1 = header1a;
   else;       header1 = header1~header1a;
   endif;
   hj = hj+1;
endo;
header2a = {"ep" "r" "im" "ex" "dp" "y"};
hj = 1;
do until hj > 4;
   if hj eq 1; header2 = header2a;
   else;       header2 = header2~header2a;
   endif;
   hj = hj+1;
endo;
header3 = {"ep" "im" "ex" "dp" "y"};
header = header0~header1~header2~header3;

@ ************************************************ @
@ Construction of trade weight-based link matrices @ 
@ ************************************************
  loading the weight matrices
  wi:  26 by 26 matrix of country's trade weights 
  wia: 26 by 26 matrix of country's trade weight allowing for 
       countries without interest rates (SAUDI)  
  wib: 26 by 26 matrix of ith country's trade weight allowing for 
       countries without equity prcies (SAU,CHN,IDO,PER and TUR) 
  ith row refers to ith country's trade weights with 25 other countries

  These trade-weights matrices are saved in the Excel file, BOK_WEIGHTS.xls

  Then constructing the individual link matrices and the stacked one
  WWi: 12 by 177 for i=0,...20, 
       11 by 177 for i=21,...,24 (CHN,IDO,PER,TUR)
       10 by 177 for i=25 (SAU)
*
* Hence the dimension of WW is 306 by 177
*
* In case when you change the model specification, you may need to re-construct 
* the link matrices carefully along the lines described in GHS. 
* ************************************************************* @
@ Loading trade-weight matrices: 26 by 26 @ 
wi  = xlsreadm("FIXED_WEIGHTS","B2:AA27",1,""); @ for eps, dps, ys @
wia = xlsreadm("FIXED_WEIGHTS","B2:AA27",2,""); @ for rs @
wib = xlsreadm("FIXED_WEIGHTS","B2:AA27",3,""); @ for qs @

ii = 1;
do until ii > 26;

   wij = wi[ii,.];   @ 1 by 26 vector of general trade weights @ 
   wija = wia[ii,.]; @ 1 by 26 vector of weights for foreign interest @
   wijb = wib[ii,.]; @ 1 by 26 vector of weights for foreign equity @

@ Constructions of the individual country link matrices, see GHS @
   if ii eq 1; @ US @
      { WWi } = link0(wij,wija,wijb); 
   elseif ii > 1 and ii < 22; @ EU to THA @
      { WWi } = link1(wij,wija,wijb,ii); 
   elseif ii > 21 and ii <26; @ CHN, INDO, PERU, TUR @
      { WWi } = link2(wij,wija,wijb,ii); 
   elseif ii eq 26; @ SAUDI @
      { WWi } = link3(wij,wija,wijb); 
   endif;


@ Exporting link matrix to Excel file @
   ll0 = xlswritem(WWi,"Linki","a1",ii,"");


@ Or saving @
   if ii eq 1; WW = WWi;
   else;       WW = WW|WWi;
   endif;      
   
   ii = ii + 1;
endo;

@ ******************************************* @
@ Individual country-sepcific VECM estimation @
@ ******************************************* @
i = 1;
do until i > n_cou;

   case = casei[i];       @ deterministic option @
   brpoint = brpointi[i]; @ knwon break point @
   rcoin = rcoini[i];     @ no. of cointegrating relations @

@  DATA LOADING @
@* 1980q2-2009q2, 117 quarterly obs: Base year 2000 @
@* Reading Excel file @
   BOK26 = xlsreadm("GVAR26_NEW","B2:R118",i,""); 
@ sample adjustment: see above @
   BOK26 = BOK26[stt0+1:rows(BOK26)-stt1,.];

@ Construction of the data for each country 
* X: Foreign trade-weight-based exogenous I(1) variables,
* Y: Domestic endogenous I(1) variables. 
* ************************************** @
   { X, Y } = DATA1(BOK26,i);

@ Number of exogenous and endogenos variables @
   xorder = cols(x);  yorder = cols(y); m = xorder + yorder; 

@ Saving global variables for forecasting, currently 176, 
* see data2 and GHS for details  @
   { GXi } = DATA2(BOK26,i);
   if i eq 1; GX = GXi;
   else;      GX = GX~GXi; @ T by m @
   endif;

@ *******************
* DATA TRANSFORMATION
* *******************
* DY: my by T, DX: mx by T, Z1: m by T, ADZLAGS: m(maxp-1) by T
* ************************************************************* @
   { DY, Z1, DX, ADZLAGS } = TRDATA(Y,X,yorder,xorder);

@ Sample adjustment: initial obs lost due to the lags @
   stt = porder;
   DY = DY[.,stt+1:cols(DY)]; Z1 = Z1[.,stt+1:cols(Z1)];
   sample = cols(dy); @ No. of observations @

   if xorder eq 0;
   else; DX = DX[.,stt+1:cols(DX)];
   endif;
   ActDX = DX; @ actual sample of DX for bootstap, where ?? @

   ADZLAGS = ADZLAGS[.,stt+1:cols(ADZLAGS)];
   if porder eq 1;
   else; DZLAGS = ADZLAGS[1:m*(porder-1),.];
   endif;

@ *******************************
* LONG-RUN COINTEGRATION ANALYSIS
* ******************************* @
@ ECM TERMS, ZS1 and all other regressors, YY
* ******************************************* @
   inpt = ones(sample,1); tr1 = seqa(0,1,sample); tr = seqa(1,1,sample); 
   inpt = inpt'; tr1 = tr1'; tr = tr';

@* Break dummies if necessary @ 
   if brpoint eq 0; brpoint = round(sample/2); @ no break @
   endif;
    
   /* Breakpoints are measured from 1980Q2 */
   br = brpoint-stt-stt0; @ break point adj in the sample deleting initial obs @
   iduma = zeros(br-1,1);
   idumb = ones(sample-br+1,1);
   idum = iduma|idumb;
   tduma = zeros(br-1,1);
   tdumb = seqa(1,1,sample-br+1);
   tdum = tduma|tdumb;

   { idum1, adidum, tdum1, adtdum} = TRDATA2(idum,tdum,stt);

   idum1 = idum1[.,stt+1:cols(idum1)];
   tdum1 = tdum1[.,stt+1:cols(tdum1)];
   adidum = adidum[.,stt+1:cols(adidum)];
   adtdum = adtdum[.,stt+1:cols(adtdum)];
   didum = adidum[1:porder,.]; @ including the contemporaneous one @
   dtdum = adtdum[1:porder,.]; @ including the contemporaneous one @

   { ZS1, YY } = CREG(case,Z1,inpt,TR1,idum1,tdum1,porder,
                 DZlags,xorder,DX,TR,didum,dtdum);

@ ********************************** @
@ COINTEGRATION TEST & Johansen beta @
@ ********************************** @
   { COTEST, JBETA } = COINTEST(DY,ZS1,YY,yorder,xorder);

/*
@ Exporting cointegration test results to Excel file @
   aa1 = {"CoinTest" "eigenval" "LRTrace" "LRMax"};
   aa2 = {"r<=0", "r<=1", "r<=2", "r<=3", "r<=4", "r<=5", "r<=6", "r<=7"};
   aaa1 = xlswritesa(aa1,  "Cointest","a1",i,"");
   aaa2 = xlswritesa(aa2,  "Cointest","a2",i,"");
   out1 = xlswritem(COTEST,"Cointest","b2",i,"");
*/

@ Exactly identified (eigen) betas for rcoin[i] @
   ms = rows(Jbeta); @ z + no. of restrcited deterministics @
   EXbeta = Jbeta[.,ms-rcoin+1:ms]; @ changing the column order?? @

@ Error correction terms: r by T @
   EXxi = EXbeta'ZS1; @ 1 by T @

/*
@* estimation of alpha and omega given Jbeta and max likelihood @
   { Jalpha, Jomega, JMLL } = AOest(EXbeta,DY,ZS1,YY,yorder,sample,rcoin);
   "i=" i; "Johansen RRR estimates";
   "beta"; EXbeta; "alpha"; Jalpha; "omega"; Jomega;
   "maximum likelihood"; JMLL;
*/

@ ****************************
* Short-run Dynamic Estimation
* **************************** @
@ ***********************
* Conditional VECM for dy
* *********************** @
   EXYreg = EXxi|YY;   @ ?? by T @
   k0 = yorder*rows(YY); @ number of total parameters @
   kex = k0 + (ms*rcoin-rcoin^2);
   { EXally, EXey, EXOmey, EXLL, EXAIC, EXSBC, EXHQ }
   = VECdy(Dy,EXYreg,sample,yorder,kEX);

@ Abeta: all beta's including restricted deterministics @
   Abeta=EXbeta; allY=EXallY; OmegaY=EXOmeY; eY=EXeY; Yreg=EXYreg;

@ ********************************
  Marginal VAR for dx: unnecessary
  ******************************** @
@* Two options:
   Case A. dx on inpt, dzlags
   Case B. dx on inpt, significant dzlags @

@* Case A: Unrestricted  @
   if     case eq 1; XregA = dzlags; 
   elseif case eq 2; XregA = dzlags;
   elseif case eq 3; XregA = dzlags|inpt; 
   elseif case eq 4; XregA = dzlags|inpt; 
   elseif case eq 5; XregA = dzlags|inpt|tr; 
   elseif case eq 6; XregA = dzlags|inpt|didum; 
   elseif case eq 7; XregA = dzlags|inpt|dtdum; @ add didum ?? @
   endif;
   { allxA, exA, OmexA, LLxA, AICxA, SBCxA, HQxA }
   = MARdx(DX,XregA);

@* Case B: Restricted: not subject to breaks @
   if     case eq 1; XregB = dzlags; 
   elseif case eq 2; XregB = dzlags;
   elseif case eq 3; XregB = dzlags|inpt; 
   elseif case eq 4; XregB = dzlags|inpt; 
   elseif case eq 5; XregB = dzlags|inpt|tr; 
   elseif case eq 6; XregB = dzlags|inpt;
   elseif case eq 7; XregB = dzlags|inpt; 
                     XregB = XregB~zeros(xorder,rows(didum)); 
   endif;
   { allxB, exB, OmexB, LLxB, AICxB, SBCxB, HQxB }
   = MARdx(DX,XregB);

   if case eq 6; 
      allX = allXB; Omegax = OmeXB; eX = eXB; Xreg=XregB;
   elseif case eq 7;  
      allX = allXB; Omegax = OmeXB; eX = eXB; Xreg=XregB;
   else;
      allX = allXA; Omegax = OmeXA; eX = eXA; Xreg=XregA;
   endif;

@ ******************************************
* Conditional VECM estimation resutls for DY
* ******************************************
* ESTRES: coefficients~standard err~t-ratio
* DIAGRES1: RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ: 7x1 
* DIAGRES2: chiSC~chiFF~chiNO~chiHE 
*            pSC  ~pFF  ~pNO  ~pHE          
* ****************************************** @ 
   VRES = eY;          @ Residuals @
   VYHAT = ALLY*Yreg;  @ dyhat @
   VREG = Yreg';       @ regressors: T by nreg @
   nreg = cols(VREG);

   bb1 = {"Coeff" "Std Err" "t-ratio" "" "R-sq" "R-Bar" "Sige" "LL" "AIC" "SBC" "HQ"};
   bb2 = {"SC" "FF" "NOR" "HE"};
   bb3 = {"Chi", "p-value"};
   ecmi = 1;
   do until ecmi > yorder;
      VDEP = DY[ecmi,.]';       @ dependent variable: T by 1 @
      RESID = VRES[ecmi,.]';    @ residual: T by 1 @
      ECMCOEFF = ALLY[ecmi,.]'; @ coefficients @
      YHAT = VYHAT[ecmi,.]';    @ fitted value: T by 1 @

      { ESTRES, DIAGRES1, DIAGRES2 } 
      = VECM(ECMCOEFF,VDEP,VREG,RESID,YHAT,nreg);

/* EXPORT CONDITIONAL VEC RESULTS */

/*
      ecmi0 = (i-1)*yorder + ecmi;
      yout1 = xlswritem(ESTRES,   "VECY","b2",ecmi0,"");
      yout2 = xlswritem(DIAGRES1',"VECY","f2",ecmi0,"");
      yout3 = xlswritem(DIAGRES2, "VECY","f5",ecmi0,"");

      bbb1 = xlswritesa(bb1,"VECY","b1",ecmi0,"");
      bbb2 = xlswritesa(bb2,"VECY","f4",ecmi0,"");
      bbb3 = xlswritesa(bb3,"VECY","e5",ecmi0,"");
*/
      ecmi = ecmi + 1;
   endo;

@ **************************************
* Marginal VAR estimation resutls for DX 
* **************************************
* ESTRES: coefficients~standard err~t-ratio 
* DIAGRES1: RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ: 7x1 
* DIAGRES2: chiSC~chiFF~chiNO~chiHE 
             pSC  ~pFF  ~pNO  ~pHE  
* ****************************************** @ 
   VRES = eX;          @ Residuals @
   VXHAT = ALLX*Xreg;  @ dyhat @
   VREG = Xreg';       @ common regressors:  @
   nreg = cols(VREG);

   ej = 1;
   do until ej > xorder;
      VARCO = ALLX[ej,.]';  @ coefficients @
      VDEP = DX[ej,.]';     @ dependent variable @
      RESID = VRES[ej,.]';  @ residual @
      XHAT = VXHAT[ej,.]';  @ fitted value @

     { ESTRES, DIAGRES1, DIAGRES2 } 
     = VECM(VARCO,VDEP,VREG,RESID,XHAT,nreg);

/* EXPORT MARGINAL VAR RESULTS */

/*
      ej0 = (i-1)*xorder + ej;
      xout1 = xlswritem(ESTRES,   "VARX","b2",ej0,"");
      xout2 = xlswritem(DIAGRES1',"VARX","f2",ej0,"");
      xout3 = xlswritem(DIAGRES2, "VARX","f5",ej0,"");

      ccc1 = xlswritesa(bb1,"VARX","b1",ej0,"");
      ccc2 = xlswritesa(bb2,"VARX","f4",ej0,"");
      ccc3 = xlswritesa(bb3,"VARX","e5",ej0,"");
*/
     ej = ej + 1;
   endo;

@* Decomposition of parameters of conditional VECM: 
   AlpY: m_i by r_i,
   GamY: m_i by (m_i+ms_i)*porder 
   Lam: m_i by ms_i
   b0Y: intercepts, m_i by 1, 
   b1Y: trend coefficients, m_i by 1
   d0Y: intercept shift dummy, m_i by 1,
   d1Y: trend shift dummy, m_i by 1,
   ddY: coeffs on delta intercept dummy, m_i by porder
   dtY: coeffs on delta trend dummy, m_i by porder
* **************************************************** @
   {Alpi,Gami,Lami,b0i,b1i,d0i,d1i,ddi,dti} 
   = decompY(ally,rcoin,porder,yorder,xorder,case,Abeta);

@ ****************************** @
@ CONSTRUCTION OF VARX PARAMTERS: now VARX(2,2) @
@ ****************************** @
   Bbeta = Abeta[1:m,.]; @ (mx+my) by r @
   Paii = Alpi*Bbeta';
   mLami = -Lami; 
   Ai0 = eye(yorder)~mLami; @ my by (my+mx) @

   if porder eq 2;
      Ai1 = Ai0 + Paii + Gami; 
      Ai2 = -Gami;
   elseif porder eq 3;
      Gami1 = Gami[.,1:m]; Gami2 = Gami[.,m+1:2*m];
      Ai1 = Ai0 + Paii + Gami1;
      Ai2 = Gami2 - Gami1;
      Ai3 = -Gami2;
   elseif porder eq 4;
      Gami1 = Gami[.,1:m]; Gami2 = Gami[.,m+1:2*m]; 
      Gami3 = Gami[.,2*m+1:3*m];
      Ai1 = Ai0 + Paii + Gami1;
      Ai2 = Gami2 - Gami1;
      Ai3 = Gami3 - Gami2;
      Ai4 = -Gami3;
   endif;

@ ****************************************************** @ 
@* Saving individual country-specific estimation results @
@ ****************************************************** 
  Bbetai: cointegrating vectors (mx+my) by r 
  residi: my by T residuals or shocks
  sigui:  my by my variance matrix for individual country shocks
* ************************************************************** @ 

/* EXPORT OTHER ESTIMATION RESULTS */
  

   bcb0 = xlswritem(Bbeta,"BETAi","a1",i,"");

   bcb1 = xlswritem(eY,"residi","a1",i,"");
   bcb2 = xlswritem(OmegaY,"sigui","a1",i,"");

   bb0 = xlswritem(b0i,"b0i","a1",i,"");
   bb1 = xlswritem(b1i,"b1i","a1",i,"");
   bd0 = xlswritem(d0i,"d0i","a1",i,"");
   bd1 = xlswritem(d1i,"d1i","a1",i,"");
   bdd = xlswritem(ddi,"ddi","a1",i,"");
   bdt = xlswritem(dti,"dti","a1",i,"");


@ alternatively saving uhat: mn by T @
  if i eq 1; uhat = eY;
  else;      uhat = uhat|eY;
  endif;

@ *******************************
* Construction of GVAR parameters
* H0, H1, H2: 176 by 177
* b0, b1, d0, d1: 176 by 1
* dd, dt: 176 by p
* see GHS for details
* ******************* @
@ loading individual country weight matrices: 12 (11 or 10) by 177 @
   wwi = xlsreadm("Linki","A1:FU12",i,""); 
   if i eq 1; @ US: my=7, mx=3, so m=10 @
      WWi = WWi[1:rows(WWi)-2,.]; @ delete the last two rows @ 
   elseif i > 21 and i <26; @ CHN, INDO, PERU, TUR, my=6, mx=5, so m=11 @
      WWi = WWi[1:rows(WWi)-1,.]; @ delete the last row @ 
   elseif i eq 26; @ SAUDI: my=5, mx=5, so m=10 @
      WWi = WWi[1:rows(WWi)-2,.]; @ delete the last two rows @ 
   endif;

   Hi0 = Ai0*WWi; @ 12 (11 or 10) by 177 @ 
   Hi1 = Ai1*WWi;
   Hi2 = Ai2*WWi;

   if i eq 1; 
      H0 = Hi0; H1 = Hi1; H2 = Hi2;
      b0 = b0i; b1 = b1i; d0 = d0i; d1 = d1i; 
      dd = ddi; dt = dti;
   else; 
      H0 = H0|Hi0; H1 = H1|Hi1; H2 = H2|Hi2;
      b0 = b0|b0i; b1 = b1|b1i; d0 = d0|d0i; d1 = d1|d1i; 
      dd = dd|ddi; dt = dt|dti;
   endif;


/* End of Looping for CVAR of 26 countries */ 
   i = i + 1;
endo;


@ Solving uniquely for the global vector x = (x0',x1',...,xN')'
  x0 = (po,r0,m0,x0,q0,p0,y0) 
  xi = (epi,ri,mi,xi,qi,dpi,yi) i=1,...,20,
  xi = (epi,ri,mi,xi,dpi,yi)    i=21,...,24,
  xi = (epi,mi,xi,dpi,yi)       i=25    
*
* See the procedure DATA2 for detals
* Then define the transformation matrices, S0, S1 carefully
* See GHS
* ******* @  
S011 = zeros(1,7)|eye(7); S011[1,6] = -1;
S012 = zeros(8,176-7); 
S021 = zeros(176-7,7); S022 = eye(176-7);
S01 = S011~S012;
S02 = S021~S022;
S0 = S01|S02;

S111 = zeros(8,7); S111[7,6] = 1;
S112 = zeros(8,176-7);
S121 = zeros(176-7,7); S122 = zeros(176-7,176-7);
S11 = S111~S112;
S12 = S121~S122;
S1 = S11|S12;

F0 = H0*S0; IF0 = Inv(F0);
F1 = H1*S0 + H0*S1;
F2 = H2*S0 - H1*S1;
F3 = -H2*S1;

@ Final GVAR parameters @
G1 = IF0*F1; G2 = IF0*F2; G3 = IF0*F3;
gb0 = IF0*b0; gb1 = IF0*b1; gd0 = IF0*d0; gd1 = IF0*d1;   
gdd = IF0*dd; gdt = IF0*dt;

@ *********************************************** @ 
@ Construction of VMA & BIGVAR1 matrices for GVAR @
@ *********************************************** @
GALL = G1~G2~G3;
GVARp = 3;  @ GVAR lag order @
nhorz = 24; @ forecast horizon @
{ BIGVAR1, BN } = DYPARA1(GALL,GVARp,nhorz);

@ ***************************** @ 
@ Construction of SIGU and SIGE @
@ *******************************
  SIGU: Covariance matrix for country specific residual ui,
  SIGE: Covariance matrix for global residual ei = Inv(F0)*ui*Inv(F0)'
  both are m by m, here m = 176  
* ***************************************************************** @ 
@ 26 by 1 vector of the number of country endo var @
mi1 = ones(21,1); mi1 = mi1.*7;  
mi2 = ones(4,1); mi2 = mi2.*6;
mi3 = 5;  
mi = mi1|mi2|mi3;
ii1 = 0; 
ii = 1;
do until ii > 26;

   ii0 = ii1+1; ii1 = ii0+mi[ii]-1;
   uhati = uhat[ii0:ii1,.]; @ mi[ii] by T @

   jj1 = 0; 
   jj = 1;
   do until jj > 26;
      jj0 = jj1+1; jj1 = jj0+mi[jj]-1;
      uhatj = uhat[jj0:jj1,.]; @ mi[jj] by T @
      siguij = uhati*uhatj'./sample; @ covariance: m[i] by m[j], dof ?? @
      if jj eq 1; sigui = siguij;
      else;       sigui = sigui~siguij; @ m[i] by m @
      endif;
      jj = jj + 1;
   endo;

   if ii eq 1; sigu = sigui;
   else;       sigu = sigu|sigui; @ m by m @
   endif;

   ii = ii + 1;
endo;
sige = IF0*sigu*IF0'; 


@ ******************************************
* PERSISTENCE PROFILES wrt global shocks, ei
* ****************************************** @
@ 26 by 1 vector of the number of country endo+exo var @
mms1 = 10;
mms2 = ones(20,1); mms2 = mms2.*12;  
mms3 = ones(4,1);  mms3 = mms3.*11;
mms4 = 10;  
mms = mms1|mms2|mms3|mms4;
ii1 = 0; 
ii = 1;
do until ii > 26;

@ individual country link matrices: 12 (11 or 10) by 177 @
   ii0 = ii1+1; ii1 = ii0+mms[ii]-1;
   wwi = WW[ii0:ii1,.]; @ mms[ii] by T @

/*   wwi = xlsreadm("Linki","A1:FU12",ii,"");  */

@ individual country cointegrating matrices: 12 (11 or 10) by ri @
   betai = xlsreadm("betai","A1:F12",ii,""); @ maximum 12 by 6 @ 
   rcoin = rcoini[ii];   @ no. of cointegrating relations @
   betai = betai[.,1:rcoin]; 


@ selecting the correct no. of country-sepcific variables @
   if ii > 21 and ii <26; @ CHN, INDO, PERU, TUR @
      WWi = WWi[1:rows(WWi)-1,.]; @ delete the last row @ 
      betai = betai[1:rows(betai)-1,.]; 
   elseif ii eq 26; @ SAUDI @
      WWi = WWi[1:rows(WWi)-2,.]; @ delete the last two rows @ 
      betai = betai[1:rows(betai)-2,.]; 
   endif;


@ selecting the correct dimension of country-sepcific betas @
   if ii eq 1; @ US: 10 variables @
      betai = betai[1:rows(betai)-2,.]; 
   elseif ii > 21 and ii <26; @ CHN, INDO, PERU, TUR: 11 variables @
      betai = betai[1:rows(betai)-1,.]; 
   elseif ii eq 26; @ SAUDI: 10 variables @
      betai = betai[1:rows(betai)-2,.]; 
   endif;

   hz = zeros(nhorz,rcoin);
   ir = 1;
   do until ir > rcoin;
      BETAir = BETAi[.,ir];
      { hz[.,ir] } = GVARPP(nhorz,BETAir,SIGE,BN,WWi,S0,S1);
      ir=ir+1;
   endo;
   ini = ones(1,rcoin); hz = ini|hz;

@* Exporting the PP results to Excel file @
   ee1 = {"PP" "CV1" "CV2" "CV3" "CV4" "CV5" "CV6"}; 
   ee2 = xlswritesa(ee1,"GVARPP","a1",ii,"");
   ee3 = xlswritem(hz,  "GVARPP","b2",ii,"");

   ii = ii + 1;
endo;


@ ****************************
* Generalised Impulse Response
* **************************** @
ij = 1;
do until ij > nshock;
   sh_op = eshock[ij];
   { GVGIR } = GVARGIR(sh_op,nhorz,SIGU,BN,IF0); 

@* Exporting the results to Excel file *@
   ogg0 = xlswritesa(CHD,   "GVARGIR","b1",ij,"");
   ogg1 = xlswritesa(header,"GVARGIR","b2",ij,"");
   ogg2 = xlswritem(GVGIR,  "GVARGIR","b3",ij,"");

/*
if ij eq 3;
   ogg2 = xlswritem(-1*GVGIR,  "GVARGIR","b3",ij,"");
else; 
   ogg2 = xlswritem(GVGIR,  "GVARGIR","b3",ij,"");
endif;
*/

   ij = ij + 1;
endo;

/*
@ *************************************************
* Generalised Forecast Error Variance Decomposition
* ************************************************* @
ij = 1;
do until ij > nvshock;
   va_op = vshock[ij];
   { GVFEVD } = GVARFEVD(va_op,nhorz,SIGU,BN,IF0); 

@* Exporting the results to Excel file *@
   ogv0 = xlswritesa(CHD,   "GVARFEVD","b1",ij,"");
   ogv1 = xlswritesa(header,"GVARFEVD","b2",ij,"");
   ogv2 = xlswritem(100.*GVFEVD, "GVARFEVD","b3",ij,""); @ % @

   ij = ij + 1;
endo;
*/

/*
@ Saving the results for probability forecast evaluation in the companion program @
save BIGVAR1 BN GALL gb0 gb1 gd0 gd1 gdd gdt SIGE SIGU GX GVARp;
save nhorz sample case brpoint;
save Lhat A4Lhat ALhat Dhat A4Dhat ADhat;
save CHD header;
*/

"Finished";  time;

END;

@* ********** *@
@* PROCEDURES *@
@* ********** *@

@ ******************************** @
  proc (1) = link0(wij,wija,wijb);
@ ********************************
  Reference country: US
  wij:  1 by 26 vector of ith country's trade weight 
  wija: 1 by 26 vector of ith country's trade weight allowing 
       for countries without interest rates (SAUDI)  
  wijb: 1 by 26 vector of ith country's trade weight allowing for 
       countries without equity prcies (CHN, IDO, PER, TUR, SAU)  
  WW: 12 by 177 
* ************************************************ @
local R1,W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,W12,W13,W14,W15,W16,
      W17,W18,W19,W20,W21,W22,W23,W24,W25,W26,WW,WA,WB;

R1 = zeros(7,1)~eye(7); 

W1 = zeros(3,8); 
W1[1,1] = wij[1]; W1[2,6] = wij[1]; W1[3,7] = wij[1]; 

W2 = zeros(3,7); 
W2[1,1] = wij[2]; W2[2,6] = wij[2]; W2[3,7] = wij[2]; 
W3 = zeros(3,7); 
W3[1,1] = wij[3]; W3[2,6] = wij[3]; W3[3,7] = wij[3]; 
W4 = zeros(3,7); 
W4[1,1] = wij[4]; W4[2,6] = wij[4]; W4[3,7] = wij[4]; 
W5 = zeros(3,7); 
W5[1,1] = wij[5]; W5[2,6] = wij[5]; W5[3,7] = wij[5]; 
W6 = zeros(3,7); 
W6[1,1] = wij[6]; W6[2,6] = wij[6]; W6[3,7] = wij[6]; 
W7 = zeros(3,7); 
W7[1,1] = wij[7]; W7[2,6] = wij[7]; W7[3,7] = wij[7]; 
W8 = zeros(3,7); 
W8[1,1] = wij[8]; W8[2,6] = wij[8]; W8[3,7] = wij[8]; 
W9 = zeros(3,7); 
W9[1,1] = wij[9]; W9[2,6] = wij[9]; W9[3,7] = wij[9]; 
W10 = zeros(3,7); 
W10[1,1] = wij[10]; W10[2,6] = wij[10]; W10[3,7] = wij[10]; 
W11 = zeros(3,7); 
W11[1,1] = wij[11]; W11[2,6] = wij[11]; W11[3,7] = wij[11]; 
W12 = zeros(3,7); 
W12[1,1] = wij[12]; W12[3,6] = wij[12]; W12[3,7] = wij[12]; 
W13 = zeros(3,7); 
W13[1,1] = wij[13]; W13[2,6] = wij[13]; W13[3,7] = wij[13]; 
W14 = zeros(3,7); 
W14[1,1] = wij[14]; W14[2,6] = wij[14]; W14[3,7] = wij[14]; 
W15 = zeros(3,7); 
W15[1,1] = wij[15]; W15[2,6] = wij[15]; W15[3,7] = wij[15]; 
W16 = zeros(3,7); 
W16[1,1] = wij[16]; W16[2,6] = wij[16]; W16[3,7] = wij[16]; 
W17 = zeros(3,7); 
W17[1,1] = wij[17]; W17[2,6] = wij[17]; W17[3,7] = wij[17]; 
W18 = zeros(3,7); 
W18[1,1] = wij[18]; W18[2,6] = wij[18]; W18[3,7] = wij[18]; 
W19 = zeros(3,7); 
W19[1,1] = wij[19]; W19[2,6] = wij[19]; W19[3,7] = wij[19]; 
W20 = zeros(3,7); 
W20[1,1] = wij[20]; W20[2,6] = wij[20]; W20[3,7] = wij[20]; 
W21 = zeros(3,7); 
W21[1,1] = wij[21]; W21[2,6] = wij[21]; W21[3,7] = wij[21]; 

W22 = zeros(3,6); 
W22[1,1]=wij[22]; W22[2,5]=wij[22]; W22[3,6]=wij[22]; 
W23 = zeros(3,6); 
W23[1,1]=wij[23]; W23[2,5]=wij[23]; W23[3,6]=wij[23]; 
W24 = zeros(3,6); 
W24[1,1]=wij[24]; W24[2,5]=wij[24]; W24[3,6]=wij[24]; 
W25 = zeros(3,6); 
W25[1,1]=wij[25]; W25[2,5]=wij[25]; W25[3,6]=wij[25]; 

W26 = zeros(3,5); 
W26[1,1]=wij[26]; W26[2,4]=wij[26]; W26[3,5]=wij[26]; 

WA = R1~zeros(7,169);
WB = W1~W2~W3~W4~W5~W6~W7~W8~W9~W10~W11~W12~W13~W14~W15~W16
    ~W17~W18~W19~W20~W21~W22~W23~W24~W25~W26;
WW = WA|WB; 

retp(WW); endp;

@ ********************************** @
  proc (1) = link1(wij,wija,wib,ii);
@ **********************************
  EU to THA
  wi:  1 by 26 vector of ith country's trade weight 
  wia: 1 by 26 vector of ith country's trade weight allowing 
       for countries without interest rates (SAUDI)  
  wib: 1 by 26 vector of ith country's trade weight allowing for 
       countries without equity prcies (CHN, IDO, PER, TUR, SAU)  
  WW: 12 by 177 
* ************************************************ @
local R1,R2,R3,R4,R5,R6,R7,R8,R9,R10,R11,R12,R13,R14,R15,R16,
      R17,R18,R19,R20,R21,R22,R23,R24,R25,R26,
      W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,W12,W13,W14,W15,W16,
      W17,W18,W19,W20,W21,W22,W23,W24,W25,W26,WW,WA,WB;

R1 = zeros(7,8); R1[1,1] = -wij[1]; 

R2 = zeros(7,7);  R2[1,1] = -wij[2];
R3 = zeros(7,7);  R3[1,1] = -wij[3];
R4 = zeros(7,7);  R4[1,1] = -wij[4];
R5 = zeros(7,7);  R5[1,1] = -wij[5];
R6 = zeros(7,7);  R6[1,1] = -wij[6];
R7 = zeros(7,7);  R7[1,1] = -wij[7];
R8 = zeros(7,7);  R8[1,1] = -wij[8];
R9 = zeros(7,7);  R9[1,1] = -wij[9];
R10 = zeros(7,7); R10[1,1] = -wij[10];
R11 = zeros(7,7); R11[1,1] = -wij[11];
R12 = zeros(7,7); R12[1,1] = -wij[12];
R13 = zeros(7,7); R13[1,1] = -wij[13];
R14 = zeros(7,7); R14[1,1] = -wij[14];
R15 = zeros(7,7); R15[1,1] = -wij[15];
R16 = zeros(7,7); R16[1,1] = -wij[16];
R17 = zeros(7,7); R17[1,1] = -wij[17];
R18 = zeros(7,7); R18[1,1] = -wij[18];
R19 = zeros(7,7); R19[1,1] = -wij[19];
R20 = zeros(7,7); R20[1,1] = -wij[20];
R21 = zeros(7,7); R21[1,1] = -wij[21];

R22 = zeros(7,6); R22[1,1] = -wij[22];
R23 = zeros(7,6); R23[1,1] = -wij[23];
R24 = zeros(7,6); R24[1,1] = -wij[24];
R25 = zeros(7,6); R25[1,1] = -wij[25];

R26 = zeros(7,5); R26[1,1] = -wij[26]; 

W1 = zeros(5,8); 
W1[1,2]=1; 
W1[2,3] = wija[1]; W1[3,6] = wijb[1]; W1[4,7] = wij[1]; W1[5,8] = wij[1]; 
W2 = zeros(5,7); 
W2[2,2] = wija[2]; W2[3,5] = wijb[2]; W2[4,6] = wij[2]; W2[5,7] = wij[2]; 
W3 = zeros(5,7); 
W3[2,2] = wija[3]; W3[3,5] = wijb[3]; W3[4,6] = wij[3]; W3[5,7] = wij[3]; 
W4 = zeros(5,7); 
W4[2,2] = wija[4]; W4[3,5] = wijb[4]; W4[4,6] = wij[4]; W4[5,7] = wij[4]; 
W5 = zeros(5,7); 
W5[2,2] = wija[5]; W5[3,5] = wijb[5]; W5[4,6] = wij[5]; W5[5,7] = wij[5]; 
W6 = zeros(5,7); 
W6[2,2] = wija[6]; W6[3,5] = wijb[6]; W6[4,6] = wij[6]; W6[5,7] = wij[6]; 
W7 = zeros(5,7); 
W7[2,2] = wija[7]; W7[3,5] = wijb[7]; W7[4,6] = wij[7]; W7[5,7] = wij[7]; 
W8 = zeros(5,7); 
W8[2,2] = wija[8]; W8[3,5] = wijb[8]; W8[4,6] = wij[8]; W8[5,7] = wij[8]; 
W9 = zeros(5,7); 
W9[2,2] = wija[9]; W9[3,5] = wijb[9]; W9[4,6] = wij[9]; W9[5,7] = wij[9]; 
W10 = zeros(5,7); 
W10[2,2]=wija[10]; W10[3,5]=wijb[10]; W10[4,6]=wij[10]; W10[5,7]=wij[10]; 
W11 = zeros(5,7); 
W11[2,2]=wija[11]; W11[3,5]=wijb[11]; W11[4,6]=wij[11]; W11[5,7]=wij[11]; 
W12 = zeros(5,7); 
W12[2,2]=wija[12]; W12[3,5]=wijb[12]; W12[4,6]=wij[12]; W12[5,7]=wij[12]; 
W13 = zeros(5,7); 
W13[2,2]=wija[13]; W13[3,5]=wijb[13]; W13[4,6]=wij[13]; W13[5,7]=wij[13]; 
W14 = zeros(5,7); 
W14[2,2]=wija[14]; W14[3,5]=wijb[14]; W14[4,6]=wij[14]; W14[5,7]=wij[14]; 
W15 = zeros(5,7); 
W15[2,2]=wija[15]; W15[3,5]=wijb[15]; W15[4,6]=wij[15]; W15[5,7]=wij[15]; 
W16 = zeros(5,7); 
W16[2,2]=wija[16]; W16[3,5]=wijb[16]; W16[4,6]=wij[16]; W16[5,7]=wij[16]; 
W17 = zeros(5,7); 
W17[2,2]=wija[17]; W17[3,5]=wijb[17]; W17[4,6]=wij[17]; W17[5,7]=wij[17]; 
W18 = zeros(5,7); 
W18[2,2]=wija[18]; W18[3,5]=wijb[18]; W18[4,6]=wij[18]; W18[5,7]=wij[18]; 
W19 = zeros(5,7); 
W19[2,2]=wija[19]; W19[3,5]=wijb[19]; W19[4,6]=wij[19]; W19[5,7]=wij[19]; 
W20 = zeros(5,7); 
W20[2,2]=wija[20]; W20[3,5]=wijb[20]; W20[4,6]=wij[20]; W20[5,7]=wij[20]; 
W21 = zeros(5,7); 
W21[2,2]=wija[21]; W21[3,5]=wijb[21]; W21[4,6]=wij[21]; W21[5,7]=wij[21]; 

W22 = zeros(5,6); 
W22[2,2]=wija[22]; W22[4,5]=wij[22]; W22[5,6]=wij[22]; 
W23 = zeros(5,6); 
W23[2,2]=wija[23]; W23[4,5]=wij[23]; W23[5,6]=wij[23]; 
W24 = zeros(5,6); 
W24[2,2]=wija[24]; W24[4,5]=wij[24]; W24[5,6]=wij[24]; 
W25 = zeros(5,6); 
W25[2,2]=wija[25]; W25[4,5]=wij[25]; W25[5,6]=wij[25]; 

W26 = zeros(5,5); 
W26[4,4]=wij[26]; W26[5,5]=wij[26]; 

if ii eq 2; 
   R2 = eye(7); W2 = zeros(5,7); @ already 0 @
elseif ii eq 3; 
   R3 = eye(7); W3 = zeros(5,7); 
elseif ii eq 3; 
   R3 = eye(7); W3 = zeros(5,7); 
elseif ii eq 4; 
   R4 = eye(7); W4 = zeros(5,7); 
elseif ii eq 5; 
   R5 = eye(7); W5 = zeros(5,7); 
elseif ii eq 6; 
   R6 = eye(7); W6 = zeros(5,7); 
elseif ii eq 7; 
   R7 = eye(7); W7 = zeros(5,7); 
elseif ii eq 8; 
   R8 = eye(7); W8 = zeros(5,7); 
elseif ii eq 9; 
   R9 = eye(7); W9 = zeros(5,7); 
elseif ii eq 10; 
   R10 = eye(7); W10 = zeros(5,7); 
elseif ii eq 11; 
   R11 = eye(7); W11 = zeros(5,7); 
elseif ii eq 12; 
   R12 = eye(7); W12 = zeros(5,7); 
elseif ii eq 13; 
   R13 = eye(7); W13 = zeros(5,7); 
elseif ii eq 14; 
   R14 = eye(7); W14 = zeros(5,7); 
elseif ii eq 15; 
   R15 = eye(7); W15 = zeros(5,7); 
elseif ii eq 16; 
   R16 = eye(7); W16 = zeros(5,7); 
elseif ii eq 17; 
   R17 = eye(7); W17 = zeros(5,7); 
elseif ii eq 18; 
   R18 = eye(7); W18 = zeros(5,7); 
elseif ii eq 19; 
   R19 = eye(7); W19 = zeros(5,7); 
elseif ii eq 20; 
   R20 = eye(7); W20 = zeros(5,7); 
elseif ii eq 21; 
   R21 = eye(7); W21 = zeros(5,7); 
endif;

WA = R1~R2~R3~R4~R5~R6~R7~R8~R9~R10~R11~R12~R13~R14~R15~R16
    ~R17~R18~R19~R20~R21~R22~R23~R24~R25~R26;
WB = W1~W2~W3~W4~W5~W6~W7~W8~W9~W10~W11~W12~W13~W14~W15~W16
    ~W17~W18~W19~W20~W21~W22~W23~W24~W25~W26;
WW = WA|WB; 

retp(WW); endp;

@ ********************************** @
  proc (1) = link2(wij,wija,wib,ii);
@ **********************************
  CHN, INDO, PERU, TUR
  wi:  1 by 26 vector of ith country's trade weight 
  wia: 1 by 26 vector of ith country's trade weight allowing 
       for countries without interest rates (SAUDI)  
  wib: 1 by 26 vector of ith country's trade weight allowing for 
       countries without equity prcies (CHN, IDO, PER, TUR, SAU)  
  WW: 11 by 177 
* ************************************************ @
local R1,R2,R3,R4,R5,R6,R7,R8,R9,R10,R11,R12,R13,R14,R15,R16,
      R17,R18,R19,R20,R21,R22,R23,R24,R25,R26,
      W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,W12,W13,W14,W15,W16,
      W17,W18,W19,W20,W21,W22,W23,W24,W25,W26,WW,WA,WB;

R1 = zeros(6,8); R1[1,1] = -wij[1]; 

R2 = zeros(6,7);  R2[1,1] = -wij[2];
R3 = zeros(6,7);  R3[1,1] = -wij[3];
R4 = zeros(6,7);  R4[1,1] = -wij[4];
R5 = zeros(6,7);  R5[1,1] = -wij[5];
R6 = zeros(6,7);  R6[1,1] = -wij[6];
R7 = zeros(6,7);  R7[1,1] = -wij[7];
R8 = zeros(6,7);  R8[1,1] = -wij[8];
R9 = zeros(6,7);  R9[1,1] = -wij[9];
R10 = zeros(6,7); R10[1,1] = -wij[10];
R11 = zeros(6,7); R11[1,1] = -wij[11];
R12 = zeros(6,7); R12[1,1] = -wij[12];
R13 = zeros(6,7); R13[1,1] = -wij[13];
R14 = zeros(6,7); R14[1,1] = -wij[14];
R15 = zeros(6,7); R15[1,1] = -wij[15];
R16 = zeros(6,7); R16[1,1] = -wij[16];
R17 = zeros(6,7); R17[1,1] = -wij[17];
R18 = zeros(6,7); R18[1,1] = -wij[18];
R19 = zeros(6,7); R19[1,1] = -wij[19];
R20 = zeros(6,7); R20[1,1] = -wij[20];
R21 = zeros(6,7); R21[1,1] = -wij[21];

R22 = zeros(6,6); R22[1,1] = -wij[22];
R23 = zeros(6,6); R23[1,1] = -wij[23];
R24 = zeros(6,6); R24[1,1] = -wij[24];
R25 = zeros(6,6); R25[1,1] = -wij[25];

R26 = zeros(6,5); R26[1,1] = -wij[26]; 

W1 = zeros(5,8); 
W1[1,2]=1; W1[2,3]=wija[1]; W1[3,6]=wijb[1]; W1[4,7]=wij[1]; W1[5,8]=wij[1]; 

W2 = zeros(5,7); 
W2[2,2] = wija[2]; W2[3,5] = wijb[2]; W2[4,6] = wij[2]; W2[5,7] = wij[2]; 
W3 = zeros(5,7); 
W3[2,2] = wija[3]; W3[3,5] = wijb[3]; W3[4,6] = wij[3]; W3[5,7] = wij[3]; 
W4 = zeros(5,7); 
W4[2,2] = wija[4]; W4[3,5] = wijb[4]; W4[4,6] = wij[4]; W4[5,7] = wij[4]; 
W5 = zeros(5,7); 
W5[2,2] = wija[5]; W5[3,5] = wijb[5]; W5[4,6] = wij[5]; W5[5,7] = wij[5]; 
W6 = zeros(5,7); 
W6[2,2] = wija[6]; W6[3,5] = wijb[6]; W6[4,6] = wij[6]; W6[5,7] = wij[6]; 
W7 = zeros(5,7); 
W7[2,2] = wija[7]; W7[3,5] = wijb[7]; W7[4,6] = wij[7]; W7[5,7] = wij[7]; 
W8 = zeros(5,7); 
W8[2,2] = wija[8]; W8[3,5] = wijb[8]; W8[4,6] = wij[8]; W8[5,7] = wij[8]; 
W9 = zeros(5,7); 
W9[2,2] = wija[9]; W9[3,5] = wijb[9]; W9[4,6] = wij[9]; W9[5,7] = wij[9]; 
W10 = zeros(5,7); 
W10[2,2]=wija[10]; W10[3,5]=wijb[10]; W10[4,6]=wij[10]; W10[5,7]=wij[10]; 
W11 = zeros(5,7); 
W11[2,2]=wija[11]; W11[3,5]=wijb[11]; W11[4,6]=wij[11]; W11[5,7]=wij[11]; 
W12 = zeros(5,7); 
W12[2,2]=wija[12]; W12[3,5]=wijb[12]; W12[4,6]=wij[12]; W12[5,7]=wij[12]; 
W13 = zeros(5,7); 
W13[2,2]=wija[13]; W13[3,5]=wijb[13]; W13[4,6]=wij[13]; W13[5,7]=wij[13]; 
W14 = zeros(5,7); 
W14[2,2]=wija[14]; W14[3,5]=wijb[14]; W14[4,6]=wij[14]; W14[5,7]=wij[14]; 
W15 = zeros(5,7); 
W15[2,2]=wija[15]; W15[3,5]=wijb[15]; W15[4,6]=wij[15]; W15[5,7]=wij[15]; 
W16 = zeros(5,7); 
W16[2,2]=wija[16]; W16[3,5]=wijb[16]; W16[4,6]=wij[16]; W16[5,7]=wij[16]; 
W17 = zeros(5,7); 
W17[2,2]=wija[17]; W17[3,5]=wijb[17]; W17[4,6]=wij[17]; W17[5,7]=wij[17]; 
W18 = zeros(5,7); 
W18[2,2]=wija[18]; W18[3,5]=wijb[18]; W18[4,6]=wij[18]; W18[5,7]=wij[18]; 
W19 = zeros(5,7); 
W19[2,2]=wija[19]; W19[3,5]=wijb[19]; W19[4,6]=wij[19]; W19[5,7]=wij[19]; 
W20 = zeros(5,7); 
W20[2,2]=wija[20]; W20[3,5]=wijb[20]; W20[4,6]=wij[20]; W20[5,7]=wij[20]; 
W21 = zeros(5,7); 
W21[2,2]=wija[21]; W21[3,5]=wijb[21]; W21[4,6]=wij[21]; W21[5,7]=wij[21]; 

W22 = zeros(5,6); 
W22[2,2]=wija[22]; W22[4,5]=wij[22]; W22[5,6]=wij[22]; 
W23 = zeros(5,6); 
W23[2,2]=wija[23]; W23[4,5]=wij[23]; W23[5,6]=wij[23]; 
W24 = zeros(5,6); 
W24[2,2]=wija[24]; W24[4,5]=wij[24]; W24[5,6]=wij[24]; 
W25 = zeros(5,6); 
W25[2,2]=wija[25]; W25[4,5]=wij[25]; W25[5,6]=wij[25]; 

W26 = zeros(5,5); 
W26[4,4]=wij[26]; W26[5,5]=wij[26]; 

if ii eq 22; 
   R22 = eye(6); W22 = zeros(5,6); @ already 0 @
elseif ii eq 23; 
   R23 = eye(6); W23 = zeros(5,6); 
elseif ii eq 24; 
   R24 = eye(6); W24 = zeros(5,6); 
elseif ii eq 25; 
   R25 = eye(6); W25 = zeros(5,6); 
endif;

WA = R1~R2~R3~R4~R5~R6~R7~R8~R9~R10~R11~R12~R13~R14~R15~R16
    ~R17~R18~R19~R20~R21~R22~R23~R24~R25~R26;
WB = W1~W2~W3~W4~W5~W6~W7~W8~W9~W10~W11~W12~W13~W14~W15~W16
    ~W17~W18~W19~W20~W21~W22~W23~W24~W25~W26;
WW = WA|WB; 

retp(WW); endp;

@ ***************************** @
  proc (1) = link3(wij,wija,wib);
@ *****************************
  Saudi Arabia: no interest and no euqity
  wi:  1 by 26 vector of ith country's trade weight 
  wia: 1 by 26 vector of ith country's trade weight allowing 
       for countries without interest rates (SAUDI)  
  wib: 1 by 26 vector of ith country's trade weight allowing for 
       countries without equity prcies (CHN, IDO, PER, TUR, SAU)  
  WW: 10 by 177 
* ************************************************ @
local R1,R2,R3,R4,R5,R6,R7,R8,R9,R10,R11,R12,R13,R14,R15,R16,
      R17,R18,R19,R20,R21,R22,R23,R24,R25,R26,
      W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,W12,W13,W14,W15,W16,
      W17,W18,W19,W20,W21,W22,W23,W24,W25,W26,WW,WA,WB;

R1 = zeros(5,8); R1[1,1] = -wij[1]; 

R2 = zeros(5,7); R2[1,1] = -wij[2];
R3 = zeros(5,7); R3[1,1] = -wij[3];
R4 = zeros(5,7); R4[1,1] = -wij[4];
R5 = zeros(5,7); R5[1,1] = -wij[5];
R6 = zeros(5,7); R6[1,1] = -wij[6];
R7 = zeros(5,7); R7[1,1] = -wij[7];
R8 = zeros(5,7); R8[1,1] = -wij[8];
R9 = zeros(5,7); R9[1,1] = -wij[9];
R10 = zeros(5,7); R10[1,1] = -wij[10];
R11 = zeros(5,7); R11[1,1] = -wij[11];
R12 = zeros(5,7); R12[1,1] = -wij[12];
R13 = zeros(5,7); R13[1,1] = -wij[13];
R14 = zeros(5,7); R14[1,1] = -wij[14];
R15 = zeros(5,7); R15[1,1] = -wij[15];
R16 = zeros(5,7); R16[1,1] = -wij[16];
R17 = zeros(5,7); R17[1,1] = -wij[17];
R18 = zeros(5,7); R18[1,1] = -wij[18];
R19 = zeros(5,7); R19[1,1] = -wij[19];
R20 = zeros(5,7); R20[1,1] = -wij[20];
R21 = zeros(5,7); R21[1,1] = -wij[21];

R22 = zeros(5,6); R22[1,1] = -wij[22];
R23 = zeros(5,6); R23[1,1] = -wij[23];
R24 = zeros(5,6); R24[1,1] = -wij[24];
R25 = zeros(5,6); R25[1,1] = -wij[25];

R26 = zeros(5,5); R26[1,1] = -wij[26]; @ 0 @
R26 = eye(5);

W1 = zeros(5,8); 
W1[1,2] = 1; W1[2,3] = wija[1]; W1[3,6] = wijb[1];
W1[4,7] = wij[1]; W1[5,8] = wij[1]; 
 
W2 = zeros(5,7); 
W2[2,2] = wija[2]; W2[3,5] = wijb[2]; W2[4,6] = wij[2]; W2[5,7] = wij[2]; 
W3 = zeros(5,7); 
W3[2,2] = wija[3]; W3[3,5] = wijb[3]; W3[4,6] = wij[3]; W3[5,7] = wij[3]; 
W4 = zeros(5,7); 
W4[2,2] = wija[4]; W4[3,5] = wijb[4]; W4[4,6] = wij[4]; W4[5,7] = wij[4]; 
W5 = zeros(5,7); 
W5[2,2] = wija[5]; W5[3,5] = wijb[5]; W5[4,6] = wij[5]; W5[5,7] = wij[5]; 
W6 = zeros(5,7); 
W6[2,2] = wija[6]; W6[3,5] = wijb[6]; W6[4,6] = wij[6]; W6[5,7] = wij[6]; 
W7 = zeros(5,7); 
W7[2,2] = wija[7]; W7[3,5] = wijb[7]; W7[4,6] = wij[7]; W7[5,7] = wij[7]; 
W8 = zeros(5,7); 
W8[2,2] = wija[8]; W8[3,5] = wijb[8]; W8[4,6] = wij[8]; W8[5,7] = wij[8]; 
W9 = zeros(5,7); 
W9[2,2] = wija[9]; W9[3,5] = wijb[9]; W9[4,6] = wij[9]; W9[5,7] = wij[9]; 
W10 = zeros(5,7); 
W10[2,2]=wija[10]; W10[3,5]=wijb[10]; W10[4,6]=wij[10]; W10[5,7]=wij[10]; 
W11 = zeros(5,7); 
W11[2,2]=wija[11]; W11[3,5]=wijb[11]; W11[4,6]=wij[11]; W11[5,7]=wij[11]; 
W12 = zeros(5,7); 
W12[2,2]=wija[12]; W12[3,5]=wijb[12]; W12[4,6]=wij[12]; W12[5,7]=wij[12]; 
W13 = zeros(5,7); 
W13[2,2]=wija[13]; W13[3,5]=wijb[13]; W13[4,6]=wij[13]; W13[5,7]=wij[13]; 
W14 = zeros(5,7); 
W14[2,2]=wija[14]; W14[3,5]=wijb[14]; W14[4,6]=wij[14]; W14[5,7]=wij[14]; 
W15 = zeros(5,7); 
W15[2,2]=wija[15]; W15[3,5]=wijb[15]; W15[4,6]=wij[15]; W15[5,7]=wij[15]; 
W16 = zeros(5,7); 
W16[2,2]=wija[16]; W16[3,5]=wijb[16]; W16[4,6]=wij[16]; W16[5,7]=wij[16]; 
W17 = zeros(5,7); 
W17[2,2]=wija[17]; W17[3,5]=wijb[17]; W17[4,6]=wij[17]; W17[5,7]=wij[17]; 
W18 = zeros(5,7); 
W18[2,2]=wija[18]; W18[3,5]=wijb[18]; W18[4,6]=wij[18]; W18[5,7]=wij[18]; 
W19 = zeros(5,7); 
W19[2,2]=wija[19]; W19[3,5]=wijb[19]; W19[4,6]=wij[19]; W19[5,7]=wij[19]; 
W20 = zeros(5,7); 
W20[2,2]=wija[20]; W20[3,5]=wijb[20]; W20[4,6]=wij[20]; W20[5,7]=wij[20]; 
W21 = zeros(5,7); 
W21[2,2]=wija[21]; W21[3,5]=wijb[21]; W21[4,6]=wij[21]; W21[5,7]=wij[21]; 

W22 = zeros(5,6); 
W22[2,2]=wija[22]; W22[4,5]=wij[22]; W22[5,6]=wij[22]; 
W23 = zeros(5,6); 
W23[2,2]=wija[23]; W23[4,5]=wij[23]; W23[5,6]=wij[23]; 
W24 = zeros(5,6); 
W24[2,2]=wija[24]; W24[4,5]=wij[24]; W24[5,6]=wij[24]; 
W25 = zeros(5,6); 
W25[2,2]=wija[25]; W25[4,5]=wij[25]; W25[5,6]=wij[25]; 

W26 = zeros(5,5); 
W26[4,4]=wij[26]; W26[5,5]=wij[26]; @ 0's @

WA = R1~R2~R3~R4~R5~R6~R7~R8~R9~R10~R11~R12~R13~R14~R15~R16
    ~R17~R18~R19~R20~R21~R22~R23~R24~R25~R26;
WB = W1~W2~W3~W4~W5~W6~W7~W8~W9~W10~W11~W12~W13~W14~W15~W16
    ~W17~W18~W19~W20~W21~W22~W23~W24~W25~W26;
WW = WA|WB; 

retp(WW); endp;


@ *********************** @
  proc (2) = DATA1(vr,j);
@ ********************************************
* Generate foreign exogenous I(1) variables, X
* and domestic endogenous I(1) variables, Y.
* Note: 
* US (j=1)
*   X = esps~dps~ys:  T by 5 
*   Y = po~r~im~ex~q~dp~yd: T by 7
* EU to THA (j=2,...,21)
*   X = po~rs~qs~dps~ys:    T by 5 
*   Y = re~r~im~ex~q~dp~yd: T by 7
* China (j=22), Indonesia (j=23), Peru (j=24), Turkey (j=25),    
*   X = po~rs~qs~dps~ys:  T by 5 
*   Y = re~r~im~ex~dp~yd: T by 6
* Saudi Arabia (j=26)
*   X = po~rs~qs~dps~ys: T by 5 
*   Y = re~r~im~ex~yd:   T by 5
*   ************************************************************** @
local  X,Y,yd,p,dp,q,e,r,h,ex,im,po,rs,qs,hs,dps,ys,es,ps,pps,ep,
       esps,ees,re;

yd = vr[.,1];  @ log real output @
p = vr[.,2];   @ log CPI @
dp = vr[.,3];  @ inflation @
q = vr[.,4];   @ log stock index @
e = vr[.,5];   @ log nominal exch. (W/$) @
r = vr[.,6];   @ ST interest rate @
h = vr[.,7];   @ log (money/output) @
ex = vr[.,8];  @ log real export @
im = vr[.,9];  @ log real import @
po = vr[.,10]; @ log oil price, Dubai @
rs = vr[.,11]; @ F. interest, trade-weight @
qs = vr[.,12]; @ log F. stock index, trade-weight @
hs = vr[.,13]; @ log F. (money/output), trade-weight @
dps = vr[.,14];@ F. inflation, trade-weight @
ys = vr[.,15]; @ log F. output, trade-weight @
es = vr[.,16]; @ log F. nominal exch., trade-weight @
ps = vr[.,17]; @ log F. CPI, trade-weight @

@* different definitions of exchange rates: all in logs @
pps = p-ps;  @ relative price @
ep = e-p;    @ real exch. @
esps = es-ps;@ F. real exch. @
ees = e-es;  @ nominal effective exchange rate @
re = ep-esps; @ real effective exchange rate @

X = po~rs~qs~dps~ys; @ T by 5 @ 
if j eq 1; @ US @
   X = esps~dps~ys; @ maybe delete rs and qs?? @
endif; 

Y = re~r~im~ex~q~dp~yd; @ T by 7 @
if j eq 1; @ US @
   Y = po~r~im~ex~q~dp~yd;
elseif j > 21 and j <26; @ CHN, INDO, PERU, TUR @
   Y = re~r~im~ex~dp~yd; @ T by 6 @
elseif j eq 26; @ SAUDI @
   Y = re~im~ex~dp~yd; @ T by 5 @
endif;

retp(X,Y);endp;

@ *********************** @
  proc (1) = DATA2(vr,j);
@ ****************************************
* Generate Global variables: T by mg, mg=176 
* US (j=1)
*   GX = po~r~im~ex~q~p~yd: T by 7
* EU to THA (j=2,...,21)
*   GX = ep~r~im~ex~q~dp~yd: T by 7
* China (j=22), Indonesia (j=23), Peru (j=24), Turkey (j=25),    
*   GX = ep~r~im~ex~dp~yd: T by 6
* Saudi Arabia (j=26)
*   GX = esps~im~ex~dp~yd: T by 5
* See GHS for details
* ******************* @
local  GXj,yd,p,dp,q,e,r,h,ex,im,po,rs,qs,hs,dps,ys,es,ps,pps,ep,
       esps,ees,re;

yd = vr[.,1];  @ log real output @
p = vr[.,2];   @ log CPI @
dp = vr[.,3];  @ inflation @
q = vr[.,4];   @ log stock index @
e = vr[.,5];   @ log nominal exch. (W/$) @
r = vr[.,6];   @ ST interest rate @
h = vr[.,7];   @ log (money/output) @
ex = vr[.,8];  @ log real export @
im = vr[.,9];  @ log real import @
po = vr[.,10]; @ log oil price, Dubai @
rs = vr[.,11]; @ F. interest, trade-weight @
qs = vr[.,12]; @ log F. stock index, trade-weight @
hs = vr[.,13]; @ log F. (money/output), trade-weight @
dps = vr[.,14];@ F. inflation, trade-weight @
ys = vr[.,15]; @ log F. output, trade-weight @
es = vr[.,16]; @ log F. nominal exch., trade-weight @
ps = vr[.,17]; @ log F. CPI, trade-weight @

@* different definitions of exchange rates: all in logs @
pps = p-ps;  @ relative price @
ep = e-p;    @ real exch. @
esps = es-ps;@ F. real exch. @
ees = e-es;  @ nominal effective exchange rate @
re = ep-esps; @ real effective exchange rate @

GXj = ep~r~im~ex~q~dp~yd;  @ T by 7 @
if j eq 1; @ US @
   GXj = po~r~im~ex~q~p~yd;  @ T by 7 @
elseif j > 21 and j <26; @ CHN, INDO, PERU, TUR @
   GXj = ep~r~im~ex~dp~yd; @ T by 6 @
elseif j eq 26; @ SAUDI @
   GXj = ep~im~ex~dp~yd;   @ T by 5 @
endif;

retp(GXj);endp;


@ ******************************************************** @
  proc (13) = save1(Ai0,Ai1,Ai2,hi0,hi1,di0,di1,ddyi,dtyi,
                    idumi,didumi,tdumi,dtdumi);
@ *********************************************
* saving individual country parameters and dummies 
* Ai0,Ai1,Ai2: VARX(2,2) parameters, my by (my+mx)
* hi0,hi1: intercepts and trend coeff: my by 1
* di0,di1: intercept and trend shift dummy coeff: my by 1
* ddyi,dty1: FD intercept and trend shift dummy coeff: my by p
* idumi,tdumi: intercept and trend shift dummy: 1 by T?
* didumi,dtdumi: FD intercept and trend shift dummy: p by T?
* ********************************************************** @

retp(Ai0,Ai1,Ai2,hi0,hi1,di0,di1,ddyi,dtyi,idumi,didumi,tdumi,dtdumi);endp;

@ ************************************* @
  proc (4) = TRDATA(Y,X,yorder,xorder);
@ ***************************************************************
*   Generate the appropriate Data matrices of DY, DX, Z1, DZLAGS,
*   to be used in the VEC model, depending on the order of VAR(p)
*   DY: First difference of endogenous I(1) variables
*   DX: First difference of exogenous I(1) variables
*   Z = Y~X
*   Z1: First lagged values of Z
*   ADZLAGS = DZ1~DZ2~...~DZ(p-1): T by m(p-1)
*   ************************************************************** @
local  MM,Y1,DY,Z,Z1,adzlags,dX,DZ,DZ1,DZ2,DZ3,DZ4,DZ5,DZ6,DZ7,DZ8,
       DZ9,DZ10,DZ11;

mm = yorder + xorder;
Y1 = LAG(Y); DY = Y - Y1;

if Xorder eq 0;   Z = Y; DX = 0; @ No exog variables: Txmy @
else;             Z = Y~X;       @ Tx(my+mx): chk @
                  DX = X - LAG(X);
endif;

Z1 = LAG(Z);     DZ = Z - Z1;    DZ1 = Lag(DZ);  DZ2 = Lag(DZ1);
DZ3 = Lag(DZ2);  DZ4 = Lag(DZ3); DZ5 = Lag(DZ4); DZ6 = Lag(DZ5);
DZ7 = Lag(DZ6);  DZ8 = Lag(DZ7); DZ9 = Lag(DZ8); DZ10 = Lag(DZ9);
DZ11 = Lag(DZ10);

@ all lagged values up to maximum lag order @
ADZLAGS = DZ1~DZ2~DZ3~DZ4~DZ5~DZ6~DZ7~DZ8~DZ9~DZ10~DZ11;

@* transpose the data for matching the specification *@
DY = DY';  DX = DX';  Z1 = Z1'; ADZLAGS = ADZLAGS';

retp(DY,Z1,DX,ADZLAGS);endp;

@ ********************************* @
  PROC (4) = TRDATA2(idum,tdum,stt);
@ ********************************* @
@ idum1: lagged intercept shift dummy
* tdum1: lagged trend shift dummy
* ADidum = Didum~Didum1~...~Didum(p-1)
* ADtdum = Dtdum~Dtdum1~...~Dtdum(p-1)
* adding initial stt observations
* ************************************ @
Local  stmp,idum1,Di0,Di1,Di2,Di3,Di4,Di5,Di6,Di7,Di8,Di9,Di10,Di11,
       tdum1,Dt0,Dt1,Dt2,Dt3,Dt4,Dt5,Dt6,Dt7,Dt8,Dt9,Dt10,Dt11;

stmp = zeros(stt,1);
idum = stmp|idum; tdum = stmp|tdum;

idum1 = LAG(idum); 
Di0 = idum - idum1; Di1 = Lag(Di0); Di2 = Lag(Di1); Di3 = Lag(Di2); 
Di4 = Lag(Di3); Di5 = Lag(Di4); Di6 = Lag(Di5); Di7 = Lag(Di6);  
Di8 = Lag(Di7); Di9 = Lag(Di8); Di10 = Lag(Di9); Di11 = Lag(Di10);
ADidum = Di0~Di1~Di2~Di3~Di4~Di5~Di6~Di7~Di8~Di9~Di10~Di11;

tdum1 = LAG(tdum); 
Dt0 = tdum - tdum1; Dt1 = Lag(Dt0); Dt2 = Lag(Dt1); Dt3 = Lag(Dt2); 
Dt4 = Lag(Dt3); Dt5 = Lag(Dt4); Dt6 = Lag(Dt5); Dt7 = Lag(Dt6);  
Dt8 = Lag(Dt7); Dt9 = Lag(Dt8); Dt10 = Lag(Dt9); Dt11 = Lag(Dt10);
ADtdum = Dt0~Dt1~Dt2~Dt3~Dt4~Dt5~Dt6~Dt7~Dt8~Dt9~Dt10~Dt11;

@* transpose the data for matching the specification *@
idum1 = idum1'; tdum1 = tdum1';
ADidum = ADidum'; ADtdum = ADtdum';

RETP(idum1,ADidum,tdum1,ADtdum);ENDP;

@ *********************************************************** @
  PROC (2) 
  = CREG(case,Z1,inpt,TR1,id1,td1,p,DZlags,mx,DX,TR,did,dtd);
@ *********************************************************** 
* CONSTRUCTION of ECM term, ZS1 and all other regressors, YY 
* ZS1: (m+dim(restricted deterministic)) by T
* YY: dim(YY) by T
* ********************************************************** @
Local zs1,yy;

IF     case eq 1; ZS1 = Z1; 
Elseif case eq 2; ZS1 = Z1|inpt;
Elseif case eq 3; ZS1 = Z1;
Elseif case eq 4; ZS1 = Z1|TR1;
Elseif case eq 6; ZS1 = Z1|TR1|id1;
Elseif case eq 7; ZS1 = Z1|TR1|td1;
ENDIF;

if p eq 1; YY = 0;
else; YY = DZLAGS;
endif;

if YY eq 0;
   if mx eq 0;
   else; YY = DX;
   endif;
else;
   if mx eq 0;
   else; YY = YY|DX;
   endif;
endif;

if YY eq 0;
   IF     case eq 3; YY = inpt;
   ELSEIF case eq 4; YY = inpt;
   ELSEIF case eq 5; YY = inpt|tr;
   ELSEIF case eq 6; YY = inpt|did;
   ELSEIF case eq 7; YY = inpt|dtd;
   ENDIF;
else;
   IF     case eq 3; YY = YY|inpt;
   ELSEIF case eq 4; YY = YY|inpt;
   ELSEIF case eq 5; YY = YY|inpt|tr;
   ELSEIF case eq 6; YY = YY|inpt|did;
   ELSEIF case eq 7; YY = YY|inpt|dtd; @ ?? @
   ENDIF;
endif;

RETP(ZS1,YY);ENDP;

@ ***************************************** @
  Proc (2) = COINTEST(DY,Z1,YY,my,mx);
@ *****************************************
* (1) OLS of DY on YY to obtain R0 and then calculate
*     S00 = (1/T)R0*R0'
* (2) OLS of Z1 on YY to obtain R1 and then calculate
*     S11 = (1/T)R1*R1',      S01 = (1/T)R0*R1'
* (3) Then, calculate
*     Bmatirx = S11
*     Amatrix = S11 - S01'Invpd(S00)*S01
* For further details see Johansen (1988, 1991) and PSS (2000)
* ************************************************************* @
local  sample,URPAI,dreg,invYY,invS00,Amatrix,Bmatrix,
       S00,S01,S11,C0,C1,R0,R1,zorder,testres,
       pp,va,ve,dd,zi,sumva,maxva,Ltrace,Lmax,sumv,cvmax,cvtrace,
       vva,vaa,im1,im2,jbeta,cc;
sample = cols(dy);

@* Z1 = (Y1 X1) and any deterministic regressors @
zorder = rows(z1);  @ dimension of z1 @

@* Computation of S00, S11 and S10 @
IF YY eq 0; R0 = DY; R1 = Z1;
ELSE;
     InvYY = Invpd(YY*YY');
     C0 = DY*YY'InvYY;
     R0 = DY - C0*YY;
     C1 = Z1*YY'InvYY;
     R1 = Z1 - C1*YY;
ENDIF;

S00 = R0*R0'./sample;
S11 = R1*R1'./sample;
S01 = R0*R1'./sample;

/* CC = invpd(S11)*s01'invpd(S00)*S01; save cc; */

@* Computation of Amatrix and Bmatrix @
InvS00 = invpd(S00);
Amatrix = S11 - S01'InvS00*S01;
Bmatrix = S11;

@* Unrestricted pi matrix @
URPAI = S01*invpd(S11);

@* Johansen Beta's using eigenvalue routine @
pp = chol(S11);
dd = inv(pp')*S01'invS00*S01*inv(pp);
{va,ve} = EIGHV(dd);

Jbeta = zeros(zorder,zorder);
zi = 1;
do until zi > zorder;
   Jbeta[.,zi] = inv(pp)*ve[.,zi];
   zi = zi + 1;
endo;

vva = zeros(zorder,1);
zi = 1;
do until zi > zorder;
   vva[zi,1] = va[zorder-zi+1,1];
   zi = zi + 1;
endo;

@* Trace and the Max Eigenvalue statistics @
vaa = zeros(zorder,1);
sumv = 0;
zi = 1;
do until zi > zorder;
   vaa[zi,1] = ln(1 - va[zi,1]);
   sumv = sumv + vaa[zi,1];
   zi = zi + 1;
endo;

sumva = zeros(my,1);
maxva = zeros(my,1);
sumva[1,1] = sumv;
maxva[1,1] = ln(1 - va[zorder,1]);
zi = 2;
do until zi > my;
   im1 = zorder + 2 - zi;
   sumva[zi,1] = sumva[zi-1,1] - vaa[im1,1];
   im2 = zorder + 1 - zi;
   maxva[zi,1] = ln(1 - va[im2,1]);
   zi = zi + 1;
endo;

Ltrace = zeros(my,1);  Lmax = zeros(my,1);
zi = 1;
do until zi > my;
   Ltrace[zi,1] = -sample*sumva[zi,1];
   Lmax[zi,1] = -sample*maxva[zi,1];
   zi = zi + 1;
endo;

VVA = VVA[1:my];
TESTRES = VVA~LTRACE~LMAX;

RETP(TESTRES,JBETA);ENDP;

@ ************************************* @
  Proc (7) = VECdy(dep,reg,sample,my,k);
@ ************************************* @
Local all,e,omega,AIC,SBC,HQ,LL;

all = dep*reg'*invpd(reg*reg'); @ parameters: my by ?? @
e = dep - all*reg;              @ residuals: my by T @
omega = e*e'./(sample-rows(reg)); @ covariance: my by my @
/* omega = e*e'./(sample-rows(reg)-1); @ Mfit: restricted trend? @ */

LL = -(sample/2)*ln(det(e*e'./sample))-(sample*my/2)*(1+ln(2*pi));
@ log-likelihood function @
AIC = LL - k;
SBC = LL - (k/2)*ln(sample);
HQ  = LL - k*ln(ln(sample));

RETP(all,e,omega,LL,AIC,SBC,HQ);ENDP;

@ ************************** @
  Proc (7) = MARdx(dep,reg);
@ ************************** @
local all,e,omega,AIC,SBC,HQ,LL,k,mx,obs;

mx = rows(dep); obs = cols(dep);
all = dep*reg'*invpd(reg*reg'); @ parameters: mx by ?? @
e = dep - all*reg;              @ residuals: mx by T @
omega = e*e'./(obs-rows(reg)); @ covariance: my by my @

if mx eq 1;
   LL = -(obs/2)*ln(e'e/obs)-(obs/2)*(1+ln(2*pi));
else;
   LL = -(obs/2)*ln(det(e*e'./obs))-(obs*mx/2)*(1+ln(2*pi));
endif;

k = rows(all)*cols(all);
AIC = LL - k;
SBC = LL - (k/2)*ln(obs);
HQ  = LL - k*ln(ln(obs));

RETP(all,e,omega,LL,AIC,SBC,HQ);ENDP;

@ ********************************************* @
  Proc (3) = VECM(ECMco,DEP,REG,RES,YHAT,nreg);
@ ********************************************* 
  Computation of estimation and diagnostic test results 
* ***************************************************** @
local diagres,sige,invreg,var,se,tratio,ti,estres;

@* computation of R2 and diagnostic statistics using VECM @
{ DIAGRES1, DIAGRES2 } = DIAGNOS(dep,reg,res,yhat,nreg);

sige = diagres1[3]^2;       @ sig^2 @
invreg = invpd(reg'reg);
var = sige.*invreg;
se = sqrt(diag(var));

tratio = zeros(rows(ecmco),1);
ti = 1;
do until ti > rows(ecmco);
   tratio[ti] = ecmco[ti]/se[ti];
   ti = ti + 1;
endo;

ESTRES = ecmco~se~tratio;

RETP(ESTRES,DIAGRES1,DIAGRES2); ENDP;

@ ****************************************** @
  PROC (2) = DIAGNOS(dep,reg,res,yhat,nreg);
@ **************************************************
* This procedure computes the followings for ECM eq:
* dy(i) = phi(i)*y(i,-1) + theta(i)*x(i) + e(i),??
* (1) R-squares and Rbar-squared
* (2) Maximized log-likelihood, AIC, SBC, HQ
* (3) Diagnostic statistics such as
* Godfrey's test of residual serial correlation: chi(4)
* Ramsey's RESET test of functional form: chi(1)
* Jarque-Bera's test of the normality: chi(2)
* Test of Heterosckedasticity: chi(1)
* see Microfit manual
* ********************************************** @
Local obs,sig2,sige,LL,AIC,SBC,HQ,Rbar,RSQ,ysq2,rbarsq,
      e,x,p,mx,w,p2,p1,num,den,mu1,e2,mu2,e3,mu3,e4,mu4,
      yhat2,re,ure,R2,tmpa,tmpb,tmpc,hdep,hconst,hindep,
      chiSC,chiFF,chiNO,chiHE,diagres1,diagres2,
      tmp2a,tmp2b,wtmp,pSC,pFF,pNO,pHE;

@* computation of the likelihood, AIC, SC, HQ and Rbar square @
obs = rows(res);
sig2 = res'res/obs;
sige = res'res/(obs-nreg);   @ variance estimate with dof @
sige = sqrt(sige);           @ standard error of regression @

LL = -(obs/2)*(1+ln(2*pi*sig2));   @ log-likelihood @

AIC = LL - nreg;                   @ Akaike @
SBC = LL - (nreg/2)*ln(obs);       @ Schwarz @
HQ  = LL - nreg*ln(ln(obs));       @ Hannan-Quinn @

ysq2 = sumc((dep-meanc(dep))^2)/obs;  @ sum of squared dmeaned dep @
RSQ = 1-(sig2/ysq2);                  @ R-squared @
RBARSQ = 1-((obs-1)/(obs-nreg))*(1-RSQ); @ adjyusted R2 @

@* computation of diagnostic test statistics using the OLS residuals @
e = res;   @ Tx1 residual @
x = reg;   @ Txk indep vble @
p = 4;     @ lags used in SC test @

@* LM test for serial correlation @
@ computing W matrix for SC @
w = zeros(obs,p);   @ Txp @
p2 = 1;
do until p2 > p;
   w[obs,p+1-p2] = e[obs-p+p2-1,1];
   p1 = 1;
   do until p1 > obs-1;
      if obs-p+p2-1-p1 > 0;
         w[obs-p1,p+1-p2] = e[obs-p+p2-1-p1,1];
      endif;
      p1 = p1 + 1;
   endo;
   p2 = p2 + 1;
endo;
wtmp = w'w - w'x*invpd(x'x)*x'w;
num = e'w*invpd(wtmp)*w'e;
den = e'e/obs;
chiSC = num/den;        @ statistic @
pSC = cdfchic(chiSC,4); @ p-value @

@* LM test for functional form @
w = yhat.^2;
wtmp = w'w - w'x*invpd(x'x)*x'w;
num = e'w*invpd(wtmp)*w'e;
chiFF = num/den;
pFF = cdfchic(chiFF,1); @ p-value @

@* LM test for normality @
mu1 = sumc(e)/obs;
e2 = e.^2;
mu2 = sumc(e2)/obs;
e3 = e.^3;
mu3 = sumc(e3)/obs;
e4 = e.^4;
mu4 = sumc(e4)/obs;
tmpa = mu3^2/(6*mu2^3);
tmpb = (mu4/mu2^2 - 3)^2;
tmpc = 3*mu1^2/(2*mu2) - (mu3*mu1/mu2^2);
chiNO = obs*(tmpa+(1/24)*tmpb) + obs*tmpc;
pNO = cdfchic(chiNO,2); @ p-value @

@* LM test for heterockedasticity @
hdep   = e2;
hconst = ones(obs,1);
yhat2 = yhat.^2;
hindep = hconst~yhat2;
ure = hdep - hindep*invpd(hindep'hindep)*hindep'hdep; @ unrestricted res @
re = hdep - hconst*invpd(hconst'hconst)*hconst'hdep;  @ restricted res @
R2 = (re're - ure'ure)/re're;
chiHE = obs*R2;
pHE = cdfchic(chiHE,1); @ p-value @

DIAGRES1 = RSQ|RBARSQ|SIGE|LL|AIC|SBC|HQ; @ 7x1 @
tmp2a = chiSC~chiFF~chiNO~chiHE; 
tmp2b = pSC~pFF~pNO~pHE; 
DIAGRES2 = tmp2a|tmp2b; @ 2x4 @

RETP(DIAGRES1,DIAGRES2);ENDP;

@ ********************************* @
  Proc (0) = PRECM(EST,DIAG1,DIAG2);
@ ********************************* @
local pval1,pval2,pval3,pval4,rr2,rr3;

"    Coeff.   Std. Err  T-ratio";"";
call printfm(est,1~1~1,"lf"~10~4|"lf"~10~4|"lf"~10~4);"";
"Diagnostic Statistics";"";
" RSQ    RBARSQ   SIGE     LL    AIC    SBC     HQ";"";
rr2 = diag1';
call printfm(rr2,1~1~1~1~1~1~1,"lf"~7~3|"lf"~7~3|"lf"~7~3|"lf"~7~2|"lf"~7~2
               |"lf"~7~2|"lf"~7~2);"";
rr3 = diag2;
"Ch-SC  Ch-FF  Ch-NO  Ch-HE";"";
call printfm(rr3,1~1~1~1,"lf"~7~2|"lf"~7~2|"lf"~7~2|"lf"~7~2);"";"";

retp;endp;

@ ******************************************** @
  Proc (9) = decompY(ally,r,p,my,mx,case,beta);
@ ******************************************** @
@ Decomposition into separate parameter blocks for VEC for dy 
* Remind: the order of regressors in VECM is:
* ecm(-1)|dzlags|DX|deterministics where
* the dimensions (row) are r, m(p-1), mx, and 1,2 or 3 
* In Cases 6 and 7, we need to conisder the coeffs on FD dummise 
* whose dimension is equal to p 
* ***************************** @
Local Alp,Gam,Lam,b0,b1,d0,d1,dd,dt,i1,i2,m;

m = my + mx;
if r eq 0;
   Alp = zeros(my,r); @ no coin @
else;
   i1 = 1; i2 = i1+r-1;
   Alp = allY[.,i1:i2];
endif;

if p eq 1; 
   Gam = 0; @ lag order @
else; 
   i1 = r+1;  i2 = i1+(p-1)*m-1;
   Gam = allY[.,i1:i2];  @ my by m(p-1)  @
endif;

if mx eq 0;  @ xorder @
   Lam = 0;
else;  
   i1 = r+(p-1)*m+1; i2 = i1+mx-1;
   Lam = allY[.,i1:i2];  @ my by mx @
endif;

b0 = zeros(my,1);  @ intercepts @
b1 = zeros(my,1);  @ trend coefficients @
d0 = zeros(my,1);  @ intercept dummy coefficients @
d1 = zeros(my,1);  @ trend dummy coefficients @
dd = zeros(my,p);  @ coefficients of FD int dum @
dt = zeros(my,p);  @ coefficients of FD tr dum @

if     case eq 2;  
       if r eq 0;
       else;  b0 = alp*beta[m+1,.]';@ restricted trends @
       endif;
elseif case eq 3;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; 
elseif case eq 4;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]'; @ restricted trends @
       endif;
elseif case eq 5;
       i1 = r+(p-1)*m+mx+1; b0 = allY[.,i1]; b1 = allY[.,i1+1];   
elseif case eq 6;
       i1 = r+(p-1)*m+mx+1; 
       b0 = allY[.,i1]; dd = allY[.,i1+1:i1+p]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]';  @ restricted trends @
             d0 = alp*beta[m+2,.]';  @ restricted int dum @
       endif;
elseif case eq 7;
       i1 = r+(p-1)*m+mx+1;
       b0 = allY[.,i1]; dt = allY[.,i1+1:i1+p]; 
       if r eq 0;
       else; b1 = alp*beta[m+1,.]';  @ restricted trends @
             d1 = alp*beta[m+2,.]';  @ restricted int dum @
       endif;
endif;

RETP(Alp,Gam,Lam,b0,b1,d0,d1,dd,dt);ENDP;

@ ************************************** @
  Proc (4) = decompX(allx,p,my,mx,case);
@ ************************************** @
@ Decomposition into separate parameter blocks for VAR for dx:
* Remind: the order of regressors in VAR is: dzlags|deterministics 
* where the dimensions (row) are m(p-1) and 1,2 or plus p. 
* In Cases 6 and 7, we need to conisder the coeffs on FD dummise 
* whose dimension is equal to p 
* ***************************** @
Local Gam,b0,b1,dd,dt,i1,i2,m;

m = my+mx;
if p eq 1; @ lag order @
   Gam = 0; 
else; 
   i1 = 1;  i2 = i1+(p-1)*m-1;
   Gam = allX[.,i1:i2];  @ mx by m(p-1)  @
endif;

b0 = zeros(mx,1);  @ intercepts @
b1 = zeros(mx,1);  @ trends @
dd = zeros(mx,p);  @ coefficients of FD int dum @
dt = zeros(mx,p);  @ coefficients of FD tr dum @

if     case eq 3;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; 
elseif case eq 4;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; 
elseif case eq 5;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; b1 = allX[.,i1+1];   
elseif case eq 6;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; dd = allX[.,i1+1:i1+p]; 
elseif case eq 7;
       i1 = (p-1)*m+1; b0 = allX[.,i1]; dt = allX[.,i1+1:i1+p]; 
endif;

retp(Gam,b0,dd,dt);endp;

@ *************************** @
  proc(2) = DYPARA1(fai,p,h);
@ ***************************
* Computation of MA parameters
* FAI: FAI(1),...,FAI(p), m by m*p
* FF: Big VAR(1) matrix: mp by mp
* BN: m by m MA matrix
* m = m_0 + ... + m_N: the number of global variables
* *************************************************** @
local  mn,ff,jsel,tmp1,tmp2,tmp,bn,btmp,bi,bi1;

mn = rows(fai); @ now 176 @ 

if p eq 1; FF = FAI; Jsel = eye(mn);
else;
   tmp1 = eye(mn*(p-1));
   tmp2 = zeros(mn*(p-1),mn);
   tmp = tmp1~tmp2;
   FF = FAI|tmp;
   Jsel = eye(mn)~zeros(mn,mn*(p-1));
endif;

@* Computation of Bn matrix (MA) @
BN = zeros(mn,h*mn);  @ MA coefficients, h=1,...,H @
btmp = eye(mn*p);
bi = 1;
do until bi > h;
   btmp = FF*btmp;
   bi1 = (bi-1)*mn+1;
   BN[.,bi1:bi1+mn-1] = Jsel*btmp*Jsel'; @ mn by mn @
   bi = bi + 1;
endo;

retp(FF,BN);endp;

@ ******************************** @
  PROC(3) = DYPARA(pai,gam,p,m,n);
@ ********************************
* Computation of MA parameters
* FAI: FAI(1),...,FAI(p), m by m*p
* FF: F matrix, mp by mp augmented matrix
* BN: m by m MA matrix
* **************************************** @
local  pgam,kk,jj,fai,ff,bn,ki,fi,ktmp1,ktmp2,ktmp3,
       Ftmp1,Ftmp2,Ftmp3,bi,bi1,btemp;

@* Computation of Fai matrix @
if p eq 1; PGAM = PAI;
else;      PGAM = PAI~GAM;   @ m by pm @
endif;

if p eq 1; KK = -eye(m); JJ = eye(m);
else;
   Ktmp1 = -eye(m*p);
   Ktmp2 = zeros(p,p);
   ki = 1;
   do until ki > p-1;
      Ktmp2[ki+1,ki] = 1;
      ki = ki + 1;
   endo;
   Ktmp3 = Ktmp2.*.eye(m);

   KK = Ktmp1 + Ktmp3;
   JJ = eye(m)~zeros(m,m*(p-1));
endif;
FAI = PGAM*KK + JJ;  @ FAI1,...,FAIp: m by m*p @

if p eq 1;  FF = FAI;
else;
   ftmp1 = FAI|zeros((p-1)*m,m*p);
   ftmp2 = zeros(p,p);
   fi = 1;
   do until fi > p-1;
      ftmp2[fi+1,fi] = 1;
      fi = fi + 1;
   endo;
   ftmp3 = ftmp2.*.eye(m);
   FF = ftmp1 + ftmp3;
endif;

@* Computation of Bn matrix (MA) @
BN = zeros(m,n*m);  @ MA coefficients @
btemp = eye(m*p);
bi = 1;
do until bi > n;
   btemp = FF*btemp;
   bi1 = (bi-1)*m+1;
   BN[.,bi1:bi1+m-1] = JJ*btemp*JJ';
   bi = bi + 1;
endo;

RETP(FAI,FF,BN);ENDP;

@ ********************************************* @
   proc (1) = GVARPP(n,BETA,OMEGA,BN,Wi,S0,S1);
@ *********************************************
* Persistence profiles of LR relations,
* hz(n) = betai'Wi*C(n)*Omega*C(n)'Wi'betai/betai'Wi*C0*Omega*C0'Wi'betai
* where 
* betai: jth beta of ith country
* Wi: link matrix of ith country
* C(n) = S0*B(n) - S1*B(n-1), C(0) = S0*B0 = S0
* B(n): VMA matrix 
* Omega is the cov mat of global shocks.
* ************************************************** @
local DENOM,mn,hz,j,j1,BNj,CN,BNjold,numer;

BETA = BETA.*1000;    @ hz is invariant to scale @
DENOM = beta'Wi*S0*OMEGA*S0'Wi'beta;

mn = rows(BN); @ dimension of global variables @
hz = zeros(n,1);
j = 1;
do until j > n;
   j1 = (j-1)*mn+1;
   BNj = BN[.,j1:j1+mn-1];
   if j eq 1; CN = S0*BNj - S1;
   else;      CN = S0*BNj - S1*BNjold;
   endif;
   numer = beta'Wi*CN*OMEGA*CN'Wi'beta;
   hz[j,1] = numer/Denom;
   BNjold = BNj;
   j = j + 1;
endo;

RETP(hz);ENDP;

@ ************************************* @
  proc (1) = GVARGIR(sh,n,SIGU,BN,IF0);
@ *************************************
* generalized impulse responses of 
* m global variables wrt jth shock
* h = 1,2,...,H and j = 1 to m.
* ******************************** @
local mn,evec,sigjj,gir0,gir,zi,j1,bj;

mn = rows(SIGU); @ dimension of global variables @
evec = zeros(mn,1);
evec[sh] = 1;   @ shock selection vector @
sigjj = evec'SIGU*evec; 
GIR0 = IF0*SIGU*evec./sqrt(sigjj); @ impact impulse: one-standard-deviation shock @
GIR = zeros(mn,n);
zi = 1;
do until zi > n;
   j1 = (zi-1)*mn + 1;
   Bj = BN[.,j1:j1+mn-1];
   GIR[.,zi] = Bj*IF0*SIGU*evec./sqrt(sigjj);
   zi = zi + 1;
endo;
GIR = GIR0~GIR;
@ transposing the results such that n+1 by mn @
GIR = GIR'; 

RETP(GIR);ENDP;

@ *************************************** @
  proc (1) = GVARFEVD(va,n,sigma,BN,IF0);
@ ***************************************
* generalized forecast error variance decomposition of 
* m global variables accounted for by jth shock innovation
* h = 1,2,...,H and j = 1 to m.
* ******************************** @
local mn,evec,sigjj,GFEVDD0,GFEVDN0,GFEVD0,GFEVD,zi,j1,Bj,
      dtmp,ntmp,dsum,nsum;

mn = rows(sigma); @ dimension of global variables @
evec = zeros(mn,1); 
evec[va] = 1;   @ variable selection vector @
sigjj = diag(sigma); @ innovation variances on diagonal, mn by 1 @
sigjj = sigjj';

GFEVDD0 = evec'IF0*sigma*IF0'evec; @ impact total variance @
GFEVDN0 = (evec'IF0*sigma)^2; 
GFEVDN0 = GFEVDN0./sigjj; @ impact contribution of mn innovations @
GFEVD0 = GFEVDN0./GFEVDD0; @ 1 by mn @

GFEVD = zeros(n,mn);
zi = 1;
do until zi > n;
   j1 = (zi-1)*mn + 1;
   Bj = BN[.,j1:j1+mn-1];
   Dtmp = evec'Bj*IF0*sigma*IF0'Bj'evec;
   Ntmp = (evec'Bj*IF0*sigma)^2;
   Ntmp = Ntmp./sigjj;
   if zi eq 1; Dsum = Dtmp; Nsum = Ntmp;
   else;       Dsum = Dsum+Dtmp; Nsum = Nsum+Ntmp;
   endif;
   GFEVD[zi,.] = Nsum./Dsum;
   zi = zi + 1;
endo;
GFEVD = GFEVD0|GFEVD;

RETP(GFEVD);ENDP;

@* ************************************************************************* *@
   proc (12) = POINTFOR(FF,BN,b0,b1,d0,d1,dd,dt,omega,ZP,Z,p,n,sample,case,br);
@* **************************************************
*  Point forecasts with standard errors 
*  ************************************ @
local m,zi,zj,tem1,tem2,tem3,sumome,bob,tem3a,jj,ZT0,asumome,
      lhat1,lhat2,lhat,lvar,alhat,alvar,llhat,llhat1,dhat,dvar,advar,adhat,
      inpt,tr,t1,gg,wt,gtmp,gw,sumL,sumD,oldbj,bj,dbj,sumdome,dbob,
      a4lhat,a4dhat,a4lvar,a4dvar,a4lstd,a4dstd,
      bjn,dbjn,a4dbjn,suma4l,suml4,suma4d,sumal,bjtmp,dbjtmp,d4bjtmp,
      a4dbob,lstd,alstd,dstd,adstd,brtr,tr1,deltad,deltat;

m = rows(BN); @ number of global variables @
@ wt: m by n of deterministic regressors (constant and trend) @
inpt = ones(n,1); tr = seqa(sample+1,1,n);
@ in Cases 4, 6, and 7, this starts from T+n-1 @
tr1  = seqa(sample,1,n);
brtr = seqa(sample-br+1,1,n);
deltad = zeros(n,p); @ unless br occurs at last samples, they are 0 @
deltat = ones(n,p);  @ unless br occurs at last samples, they are 1 @
inpt = inpt'; tr = tr';  tr1 = tr1'; brtr = brtr';
deltad = deltad'; deltat = deltat';

if     case eq 1; gg = b0;    wt = inpt;
elseif case eq 2; gg = b0;    wt = inpt;
elseif case eq 3; gg = b0;    wt = inpt;
elseif case eq 4; gg = b0~b1; wt = inpt|tr1;
elseif case eq 5; gg = b0~b1; wt = inpt|tr;
elseif case eq 6; gg = b0~b1~d0~dd; 
                  wt = inpt|tr1|inpt|deltad;
elseif case eq 7; gg = b0~b1~d1~dt;
                  wt = inpt|tr1|brtr|deltat;
endif;

zi = 1;
do until zi > n;
   gtmp = gg*wt[.,zi];
   if zi eq 1; GW = gtmp;     @ m by 1 @
   else;       GW = GW|gtmp;  @ mn by 1 @
   endif;
   zi = zi + 1;
endo;

@ slection matrix @
if p eq 1; JJ = eye(m);
else;      JJ = eye(m)~zeros(m,m*(p-1));
endif;

@ Forecasts of the level of the variables and their s.e @
Lhat1 = zeros(m,n); Lhat2 = zeros(m,n); Lhat = zeros(m,n);
tem1 = eye(m*p);
zi = 1;
do until zi > n;
   tem1 = FF*tem1;
   tem2 = JJ*tem1*ZP;
   Lhat1[.,zi] = tem2;

   if zi eq 1; tem3 = eye(m);
   else;       tem3 = BJ~tem3;
   endif;

   if     zi < n;  tem3a = tem3~zeros(m,m*(n-zi));
   elseif zi eq n; tem3a = tem3;
   endif;

   if zi eq 1;
   else;       oldBj = Bj; @ B(j-1) @
   endif;

   Bj = JJ*tem1*JJ';
   Lhat2[.,zi] = tem3a*GW;
   Lhat[.,zi] = Lhat1[.,zi] + Lhat2[.,zi];

   zi = zi + 1;
endo;

bjn = eye(m)~bn;         @ m by (n+1)*m @
dbjn = zeros(m,m*(n+1)); @ first-difference @
dbjn[.,1:m] = eye(m);
if n < 1;
else;
   zi = 2;
   do until zi > n+1;
      dbjn[.,(zi-1)*m+1:zi*m]
      = bjn[.,(zi-1)*m+1:zi*m] - bjn[.,(zi-1-1)*m+1:(zi-1)*m];
      zi = zi + 1;
   endo;
endif;

a4dbjn = zeros(m,m*(n+1));     @ 4th-difference @
if     n eq 1; a4dbjn[.,1:2*m] = bjn[.,1:2*m];
elseif n eq 2; a4dbjn[.,1:3*m] = bjn[.,1:3*m];
elseif n eq 3; a4dbjn[.,1:4*m] = bjn[.,1:4*m];
else;          a4dbjn[.,1:4*m] = bjn[.,1:4*m];
   zi = 5;
   do until zi > n+1;
      a4dbjn[.,(zi-1)*m+1:zi*m]
      = bjn[.,(zi-1)*m+1:zi*m] - bjn[.,(zi-1-4)*m+1:(zi-4)*m];
      zi = zi + 1;
   endo;
endif;

@ Standard errors @
Lvar = zeros(m,n);     ALvar = zeros(m,n);
A4Lvar = zeros(m,n);   A4Dvar = zeros(m,n);
Dvar = zeros(m,n);     ADvar = zeros(m,n);

suml = zeros(m,m);
suma4l = zeros(m,m);
suml4  = zeros(m,4*m);
sumal = zeros(m,m);
sumd = zeros(m,m);
suma4d = zeros(m,m);
zi = 1;
do until zi > n;
   bjtmp = Bjn[.,(zi-1)*m+1:zi*m];
   BOB = bjtmp*omega*bjtmp';
   suml = suml + bob;
   Lvar[.,zi] = diag(suml);
   if zi < 5;  suma4l = suma4l + suml;
               suml4[.,(zi-1)*m+1:zi*m] = suml;
   else;       suma4l = suma4l + suml;
               suma4l = suma4l - suml4[.,1:m];
               suml4 = suml4[.,m+1:4*m]~suml;
   endif;
   A4Lvar[.,zi] = diag(suma4l)./16;  @ ??? @

   sumal = sumal + suml;
   ALvar[.,zi] = diag(sumal)./zi^2;

   dbjtmp = DBjn[.,(zi-1)*m+1:zi*m];
   DBOB = dbjtmp*omega*dbjtmp';
   sumd = sumd + dbob;
   Dvar[.,zi] = diag(sumd);

   d4bjtmp = a4dBjn[.,(zi-1)*m+1:zi*m];
   a4dBOB = d4bjtmp*omega*d4bjtmp';
   suma4d = suma4d + a4dbob;
   A4Dvar[.,zi] = diag(suma4d)./16;
   ADvar[.,zi] = Lvar[.,zi]./zi^2;

   zi = zi + 1;
endo;

@ alignment @
Lhat = Lhat'; Lvar = Lvar'; ALvar = ALvar'; A4Lvar = A4Lvar';
Dvar = Dvar'; ADvar = ADvar'; A4Dvar = A4Dvar';

@ standard deviations @
Lstd = sqrt(Lvar);  ALstd = sqrt(ALvar); A4Lstd = sqrt(A4Lvar);
Dstd = sqrt(Dvar);  ADstd = sqrt(ADvar); A4Dstd = sqrt(A4Dvar);

@ Forecasts of annual average, average: level @
{ a4lhat, alhat } = Elevel(lhat,z,n,m);

@ Forecasts of quarterly, annual average, average: change @
{ dhat, a4dhat, adhat } = Echange(lhat,z,n,m);

RETP(Lhat,A4Lhat,ALhat,Dhat,A4Dhat,ADhat,Lstd,A4Lstd,ALstd,Dstd,A4Dstd,ADstd);
ENDP;

/* ****************************** */
   Proc (2) = Elevel(lhat,zd,n,m);
/* ****************************** */
@ Forecasts of 4-quarter MA and average over n @
local a4hat,ahat,zi,zt4,suml,a4tmp,zj,ztmp,obs;

A4hat = zeros(n,m);
obs = rows(zd);
zt4 = zd[obs-3:obs,.];  @ initial 4 by m @
Ahat = zeros(n,m);  suml = zeros(1,m);

zi = 1;
do until zi > n;
   zt4 = zt4[2:4,.]|Lhat[zi,.];

   A4tmp = zeros(1,m);
   zj = 1;
   do until zj > m;

      ztmp = zt4[.,zj];
      A4tmp[zj] = meanc(ztmp);
      zj = zj + 1;
   endo;
   A4hat[zi,.] = A4tmp;
   sumL = sumL + Lhat[zi,.];
   Ahat[zi,.] = sumL./zi;

   zi = zi + 1;
endo;

RETP(a4hat,ahat); ENDP;

/* ******************************* */
   Proc (3) = Echange(lhat,zd,n,m);
/* ******************************* */
local zi,ZT0,zt4,llhat,llhat1,llhat2,llhat3,llhat4,dhat,d4hat,adhat,sumd,obs;
@ Forecasts of the first-differences @
obs = rows(zd);
ZT0 = Zd[obs,.];
LLhat = ZT0|Lhat;
LLhat1 = lag(LLhat);
Dhat = LLhat - LLhat1;
Dhat = Dhat[2:rows(Dhat),.];  @ n by m matrix of forecasts of growth @

@ Forecasts of the fourth-difference divided by 4: annual rate @
zt4 = zd[obs-3:obs,.];  @ ?? z at time T, T-1, T-2, T-3 @
LLhat  = zt4|Lhat;      @ n+4 by m @
LLhat1 = lag(LLhat);
LLhat2 = lag(LLhat1);
LLhat3 = lag(LLhat2);
LLhat4 = lag(LLhat3);
D4hat = LLhat - LLhat4;
D4hat = D4hat[5:rows(D4hat),.];
D4hat = D4hat./4;

@ Forecasts of the averages @
ADhat = zeros(n,m);  sumD = zeros(1,m);
zi = 1;
do until zi > n;
   sumD = sumD + Dhat[zi,.];
   Adhat[zi,.] = sumD./zi;
   zi = zi + 1;
endo;

RETP(dhat,d4hat,adhat);ENDP;





